@@ -8,9 +8,26 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.12 $
+// $Revision: 3.17 $
 //
 // $Log: Starter.h,v $
+// Revision 3.17  2006/06/13 19:39:35  pascal_verdier
+// Minor changes.
+//
+// Revision 3.16  2006/06/05 07:20:36  pascal_verdier
+// Server startup is now delayed (with timeout) in a startup level.
+// New state MOVING added.
+// At startup, starter loop until properties have been read.
+//
+// Revision 3.15  2006/04/24 07:06:27  pascal_verdier
+// A thread is started for each level when at servers startup.
+//
+// Revision 3.14  2006/02/09 11:59:18  pascal_verdier
+// A ping thread is now started for each server.
+//
+// Revision 3.13  2006/02/08 07:13:55  pascal_verdier
+// Minor changes.
+//
 // Revision 3.12  2005/09/30 12:01:50  pascal_verdier
 // RunningServers and StoppedServers spectrum attribute max_x set to 200.
 //
@@ -110,15 +127,29 @@
 //using namespace std;
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.12 $
+ * @version	$Revision: 3.17 $
  */
 
  //	Add your own constants definitions here.
  //-----------------------------------------------
-#define PING_TIMEOUT			500
-#define	TIME_BETWEEN_STARTUPS	500
+#define PING_TIMEOUT			3	//	seconds
+#define	TIME_BETWEEN_STARTUPS	500	//	Milli seconds
 #define NOTIFY_DAEMON_SCRIPT	"notify_daemon"
 
+//	Used onlyduring the Cpp Api bug fixing on 
+//	specAtt->get_max_x() method.
+#define NB_SERVERS_MAX			200
+
+//	Structure to difine a new process to start
+typedef struct {
+	bool	could_start;
+	char	*adminname;
+	char	*servname;
+	char	*instancename;
+	char	*logfile;
+}
+NewProcess;
+
 namespace Starter_ns
 {
 
@@ -130,9 +161,10 @@ namespace Starter_ns
 
 /*
  *	Device States Description:
- *	Tango::UNKNOWN :	This device server has NOT been correctly initialised.
- *	Tango::ON :	All controlled servers are running.
- *	Tango::ALARM :	At least one controled device server is not running.
+ *  Tango::UNKNOWN :  This device server has NOT been correctly initialised.
+ *  Tango::ON :       All controlled servers are running.
+ *  Tango::ALARM :    At least one controled device server is not running.
+ *  Tango::MOVING :   The Sarter is starting servers.
  */
 
 
@@ -153,22 +185,10 @@ public :
 	 *	StarterUtil instance to be used everywhere.
 	 */
 	StarterUtil	*util;
-	/**
-	 *	State polling thread shared data.
-	 */
-	SharedData		*shared;
-	/**
-	 *	State polling thread.
-	 */
-	PollingState	*poller;
-	/**
-	 *	Polling thread ID
-	 */
-	omni_thread			*polling_id;
 	/**
 	 *	Structures defining servers to be controlled
 	 */
-	vector<ServerControled>		servers;
+	vector<ControledServer>		servers;
 	/**
 	 *	Exception must be or not thrown.
 	 */
@@ -190,6 +210,15 @@ public :
 	 */
 	Tango::DevShort	fireFromDbase;
 	bool	do_update_from_db;
+	/**
+	 *	Manage levels thread
+	 *	To be shure that (level) threads are not concurent
+	 */
+	vector<int>	start_process_thread_levels;
+	/**
+	 *	Starter is MOVING (starting servers) when > 0;
+	 */
+	int	starting_processes;
 
 	//	Here is the Start of the automatic code generation part
 	//-------------------------------------------------------------	
@@ -230,6 +259,10 @@ public :
  *	Time to wait before two startup levels in seconds.
  */
 	Tango::DevLong	interStartupLevelWait;
+/**
+ *	Timeout on device server startup in seconds.
+ */
+	Tango::DevLong	serverStartupTimeout;
 //@}
 
 /**@name Constructors
@@ -265,7 +298,7 @@ public :
 /**
  * The object desctructor.
  */	
-	~Starter() {};
+	~Starter();
 //@}
 
 	
@@ -347,10 +380,6 @@ public :
  *	Execution allowed for UpdateServersInfo command.
  */
 	virtual bool is_UpdateServersInfo_allowed(const CORBA::Any &any);
-/**
- *	Execution allowed for StartPollingThread command.
- */
-	virtual bool is_StartPollingThread_allowed(const CORBA::Any &any);
 /**
  *	Execution allowed for NotifyDaemonState command.
  */
@@ -416,11 +445,6 @@ public :
  *	@exception DevFailed
  */
 	void	update_servers_info();
-/**
- * Start the polling thread for DevState command.
- *	@exception DevFailed
- */
-	void	start_polling_thread();
 /**
  * Returns the Notify Daemon state.
  *	@return	Tango::ON if Notify daemon is running else Tango::FAULT.
@@ -452,44 +476,53 @@ protected :
  *
  *	@param devname	Device name.
  */
-const bool is_running(ServerControled *);
+const bool is_running(ControledServer *);
 /**
  *	Allocate and fill the servers controled object
  */
 void build_server_ctrl_object(void);
-/**
- *	Add the object to polling list.
- */
-void add_obj_polling(string type, string name, int period);
 /**
  *	Return how many servers to start for specified level.
  */
 int	nb_servers_to_start(int level);
-
+/**
+ *	Check if a process could be started (file exists, is not running, ...)
+ */
+NewProcess *processCouldStart(char *);
+/**
+ *	Start a thread to start processes
+ */
+void startProcesses(vector<NewProcess *>, int level);
 //@}
 };
 
-#ifdef WIN32
+
+
+
 /**
  *	Create a thread to fork a sub-process (new device server)
  */
-class StartWinProcess: public omni_thread
+class StartProcessThread: public omni_thread
 {
-	char	*server;
-	char	*instancename;
-	char	*log_file;
+	vector<NewProcess *>	processes;
+	Starter		*starter;
+	int			thread_level;
+
 public:
 /**
  *	Initialize the sub process parameters (name, domain, log_file).
  */
-	void initialize(char *, char *, char *);
+	StartProcessThread(vector<NewProcess *> v_np, int level, Starter *st);
 /**
  * Execute the fork of the sub process in a thread.
  */
 	void *run_undetached(void *);
 	void start() {start_undetached();}
+/**
+ *	Start one process
+ */
+ 	void start_process(NewProcess *);
 };
-#endif
 
 }	//	namespace
 
