@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attr_proxy.cpp,v 1.4 2005/05/20 15:19:23 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: attr_proxy.cpp,v 1.5 2006/04/13 13:18:46 taurel Exp $\n$Name:  $";
 //
 // attr_proxy.cpp 	- C++ source code file for TANGO attribute proxy api 
 //
@@ -79,7 +79,9 @@ void AttributeProxy::real_constructor (string &name)
 		}
 		else
 		{
-			dev_proxy = new DeviceProxy(device_name);
+			string noenv_dev_name(db_host);
+			noenv_dev_name = noenv_dev_name + ":" + db_port + "/" + device_name;
+			dev_proxy = new DeviceProxy(noenv_dev_name);
 			db_attr = new DbAttribute(attr_name,device_name,db_host,db_port);
 		}
 	}
@@ -107,6 +109,9 @@ void AttributeProxy::real_constructor (string &name)
 	catch (Tango::CommunicationFailed &) {}
 	catch (Tango::DevFailed &dfe)
 	{
+		delete db_attr;
+		delete dev_proxy;
+		
 		if (strcmp(dfe.errors[0].reason.in(),"API_AttrNotFound") == 0)
 		{
 			TangoSys_OMemStream desc;
@@ -120,6 +125,100 @@ void AttributeProxy::real_constructor (string &name)
 	
 }
 
+
+void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
+{
+
+//
+// First copy DeviceProxy object
+//
+
+	dev_proxy = new DeviceProxy();
+	*dev_proxy = *dev_ptr;
+
+//
+// Init local data members from device proxy object
+//
+	
+	dbase_used = dev_proxy->dbase_used;
+	from_env_var = dev_proxy->from_env_var;
+	host = dev_proxy->host;
+	port = dev_proxy->port;
+	port_num = dev_proxy->port_num;
+	db_host = dev_proxy->db_host;
+	db_port = dev_proxy->db_port;
+	db_port_num = dev_proxy->db_port_num;
+	
+	attr_name = att_name;
+
+//
+// Now AttributeProxy members
+//
+	
+	device_name = dev_proxy->device_name;
+			
+	if (dbase_used == true)
+	{
+		if (from_env_var == true)
+		{	
+			ApiUtil *ui = ApiUtil::instance();
+			if (ui->in_server() == true)
+			{
+				db_attr = new DbAttribute(attr_name,device_name,Tango::Util::instance()->get_database());
+			}
+			else
+			{
+				db_attr = new DbAttribute(attr_name,device_name);
+			}
+		}
+		else
+		{
+			db_attr = new DbAttribute(attr_name,device_name,db_host,db_port);
+		}
+	}
+	
+	ext = new AttributeProxyExt();
+	
+//
+// Check that the device support this attribute
+//
+
+	try
+	{
+		dev_proxy->get_attribute_config(attr_name);
+	}
+	catch (Tango::ConnectionFailed &) {}
+	catch (Tango::CommunicationFailed &) {}
+	catch (Tango::DevFailed &dfe)
+	{
+		delete db_attr;
+		delete ext;
+		delete dev_proxy;
+		
+		if (strcmp(dfe.errors[0].reason.in(),"API_AttrNotFound") == 0)
+		{
+			TangoSys_OMemStream desc;
+			desc << "Attribute" << attr_name << " is not supported by device " << device_name << ends;
+			
+			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedAttribute",
+						desc.str(),
+						(const char*)"AttributeProxy::ctor_from_dp()");
+		}
+	}
+}
+
+AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,const char *att_name):ext(NULL)
+{
+	string att_na(att_name);
+	ctor_from_dp(dev_ptr,att_na);
+}
+
+AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,string &att_name):ext(NULL)
+{
+	ctor_from_dp(dev_ptr,att_name);
+}
+
+
 //-----------------------------------------------------------------------------
 //
 // AttributeProxy::AttributeProxy() - copy constructor
@@ -159,8 +258,10 @@ AttributeProxy::AttributeProxy(const AttributeProxy &prev)
 			}
 			else
 			{
+				string noenv_dev_name(db_host);
+				noenv_dev_name = noenv_dev_name + ":" + db_port + "/" + device_name;
+				dev_proxy = new DeviceProxy(noenv_dev_name);
 				db_attr = new DbAttribute(attr_name,device_name);
-				dev_proxy = new DeviceProxy(device_name);
 			}
 		}
 		else
@@ -228,8 +329,10 @@ AttributeProxy &AttributeProxy::operator=(const AttributeProxy &rval)
 		}
 		else
 		{
+			string noenv_dev_name(db_host);
+			noenv_dev_name = noenv_dev_name + ":" + db_port + "/" + device_name;
+			dev_proxy = new DeviceProxy(noenv_dev_name);
 			db_attr = new DbAttribute(attr_name,device_name,db_host,db_port);
-			dev_proxy = new DeviceProxy(device_name);
 		}
 	}
 
@@ -713,6 +816,30 @@ string AttributeProxy::status()
 	return(dev_proxy->status());
 }
 
+//-----------------------------------------------------------------------------
+//
+// AttributeProxy::set_transparency_reconnection() - Set transparency
+// reconnection on the underlying device
+//
+//-----------------------------------------------------------------------------
+
+void AttributeProxy::set_transparency_reconnection(bool val)
+{
+	dev_proxy->set_transparency_reconnection(val);
+}
+
+//-----------------------------------------------------------------------------
+//
+// AttributeProxy::get_transparency_reconnection() - Get underlying device 
+// transparency reconnection flag
+//
+//-----------------------------------------------------------------------------
+
+bool AttributeProxy::get_transparency_reconnection()
+{
+	return dev_proxy->get_transparency_reconnection();
+}
+
 //-----------------------------------------------------------------------------
 //
 // AttributeProxy::get_property() - get a property from the database
