@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_base.cpp,v 3.13 2005/06/29 08:32:21 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_base.cpp,v 3.15 2006/08/11 13:51:57 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO dbapi class Database
@@ -12,6 +12,12 @@ static const char *RcsId = "$Id: dbapi_base.cpp,v 3.13 2005/06/29 08:32:21 taure
 // version 	- 1.0
 //
 // $Log: dbapi_base.cpp,v $
+// Revision 3.15  2006/08/11 13:51:57  taurel
+// - Added the Database::get_device_property_list() method
+//
+// Revision 3.14  2006/06/28 14:38:06  taurel
+// - Fix memory leak in Database::delete_device method
+//
 // Revision 3.13  2005/06/29 08:32:21  taurel
 // - Last commit before release 5.2 ?
 //
@@ -805,7 +811,7 @@ void Database::delete_device(string dev)
 {
 	Any send;
 	Any_var received;
-	send <<= string_dup(dev.c_str());
+	send <<= dev.c_str();
 	if (filedb != 0)
 		received = filedb->DbDeleteDevice(send);
 	else
@@ -2341,4 +2347,40 @@ DbDatum Database::get_attribute_alias_list(string &alias)
 	return db_datum;
 }
 
+//-----------------------------------------------------------------------------
+//
+// Database::get_device_property_list() - public method to get a device property 
+//					  list from the Database
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_device_property_list(string &dev, string &wildcard)
+{
+	Any send;
+	Any_var received;
+	DevVarStringArray *sent_names = new DevVarStringArray;
+	sent_names->length(2);
+	(*sent_names)[0] = string_dup(dev.c_str());
+	(*sent_names)[1] = string_dup(wildcard.c_str());
+	
+	send <<= sent_names;
+
+	received = command_inout("DbGetDevicePropertyList",send);
+	
+	const DevVarStringArray *prop_list;
+	received.inout() >>= prop_list;
+
+	DbDatum db_datum;
+	int n_props;
+	n_props = prop_list->length();
+	db_datum.name = dev;
+	db_datum.value_string.resize(n_props);
+	for (int i=0; i<n_props; i++)
+	{
+		db_datum.value_string[i] = (*prop_list)[i]; 
+	}
+		
+	return db_datum;
+}
+
 } // End of Tango namespace
