@@ -1,4 +1,4 @@
-//=============================================================================	
+///=============================================================================	
 //
 // file :		Attribute.h
 //
@@ -11,9 +11,55 @@
 //
 // author(s) :		A.Gotz + E.Taurel
 //
-// $Revision: 3.12 $
+// $Revision: 3.24 $
 //
 // $Log: attribute.h,v $
+// Revision 3.24  2006/06/29 11:51:12  jensmeyer
+// Moved init_poll_ext_trig() and init_poll_period() from Attribute to
+// DeviceImpl.
+//
+// Revision 3.23  2006/06/28 14:46:41  taurel
+// - Give a default value to the some of the AttributeExt prev_change_event data member which are used in the Attribute::fire_change_event() without being initialized (Valgrind detection)
+//
+// Revision 3.22  2006/06/27 17:24:08  jensmeyer
+// Corrected method descriptions.
+//
+// Revision 3.21  2006/06/20 08:58:47  jensmeyer
+// Added methods to store the polling configuration of attribute to the database when they are set in the code. The database should be a coherent reference of this configuration.
+// The methods are init_poll_ext_trig() and init_poll_period().
+//
+// Revision 3.20  2006/06/06 10:44:05  taurel
+// - Modified the DeviceImpl::push_event() method used to send user event.
+// It is now coherent with the other events type (change and archhive)
+//
+// Revision 3.19  2006/05/23 12:04:37  jensmeyer
+// Corrected method headers.
+//
+// Revision 3.18  2006/05/17 12:14:32  jensmeyer
+// Implemented manual event fireing for state and status attributes.
+//
+// Revision 3.17  2006/04/27 13:44:52  taurel
+// - Fix compatibility bug introduced in previous release by moving new
+// class data members in their corresponding extension classes
+//
+// Revision 3.16  2006/04/13 06:12:28  jensmeyer
+// Added methods to configure and fire manually change and archive events without
+// polling.
+//
+// Revision 3.15  2006/03/20 15:08:59  jensmeyer
+// Casted time_t values for porting to VC8
+//
+// Revision 3.14  2005/12/16 17:34:50  jensmeyer
+// Added methods to fire change events manually.
+//
+// Revision 3.13  2005/11/09 09:43:57  jensmeyer
+// Stopped automatic polling when subscribing for events!
+// Added exceptions when polling is not started for an attribute.
+//
+// Changed the default of the archiving period from 10 seconds to
+// no periodic event!
+// Added exception when trying to subscribe a not configured attribute.
+//
 // Revision 3.12  2005/07/28 07:34:41  taurel
 // - Fix some incompatibility between 5.1 and 5.2
 //
@@ -215,13 +261,17 @@ public:
 	long			idx_in_attr;			// Index in MultiClassAttribute vector
 	string			d_name;				// The device name
 	DeviceImpl 		*dev;				// The device object
-};
+	bool			change_event_implmented;	// Flag true if a manual fire change event is implemented.
+	bool			archive_event_implmented;	// Flag true if a manual fire archive event is implemented.	
+	bool			check_change_event_criteria;	// True if change event criteria should be checked when sending the event
+	bool			check_archive_event_criteria;	// True if change event criteria should be checked when sending the event		
+	};
 
 /**
  * This class represents a Tango attribute.
  *
- * $Author: taurel $
- * $Revision: 3.12 $
+ * $Author: jensmeyer $
+ * $Revision: 3.24 $
  */
  
 class Attribute
@@ -321,6 +371,12 @@ public:
  * @return A bitset. Each bit is set if the coresponding alarm is on
  */
 	bitset<numFlags> &is_alarmed() {return alarm_conf;}
+/**
+ * Check if the attribute is polled .
+ *
+ * @return A boolean set to true if the attribute is polled.
+ */
+	bool is_polled();
 /**
  * Check if the attribute read value is below/above the alarm level.
  *
@@ -399,7 +455,7 @@ public:
  *
  * @param The attribute date
  */
-	void set_date(struct _timeb &t) {when.tv_sec=t.time;when.tv_usec=(t.millitm*1000);when.tv_nsec=0;}
+	void set_date(struct _timeb &t) {when.tv_sec=(long)t.time;when.tv_usec=(t.millitm*1000);when.tv_nsec=0;}
 #endif
 /**
  * Set attribute date
@@ -412,7 +468,7 @@ public:
  *
  * @param The attribute date
  */
-	void set_date(time_t new_date) {when.tv_sec=new_date;when.tv_usec=0;when.tv_nsec=0;}
+	void set_date(time_t new_date) {when.tv_sec=(long)new_date;when.tv_usec=0;when.tv_nsec=0;}
 /**
  * Get attribute label property
  *
@@ -429,9 +485,9 @@ public:
  * Set attribute data quality
  *
  * @param qua	The new attribute data quality
- * @param send_event Boolean set to true if quality change event should be sent
+ * @param send_event Boolean set to true if a change event should be sent
  */
-	void set_quality(Tango::AttrQuality qua, bool send_event=true);
+	void set_quality(Tango::AttrQuality qua, bool send_event=false);
 		
 /**
  * Get attribute data size
@@ -1133,14 +1189,153 @@ public:
 				    bool release = false);
 #endif
 
+/**
+ * Fire a change event for the attribute value. The event is pushed to the notification
+ * daemon.
+ * The attibute data must be set with one of the Attribute::set_value or 
+ * Attribute::setvalue_date_quality methods before fireing the event.
+ * The event is triggered with or without the change event criteria depending
+ * on the configuration choosen with set_change_event().
+ * ATTENTION: The couple set_value() and fire_change_event() needs to be protected
+ * against concurrent accesses to the same attribute. Such an access might happen during 
+ * a synchronous read or by a reading from the polling thread.
+ * Inside all methods reading or writing commands and attributes this protection is
+ * automatically done by the Tango serialisation monitor.
+ * When fireing change events in your own code, you should use the push_change_event
+ * methods of the DeviceImpl class or protect your code with the
+ * Tango::AutoTangoMonitor on your device.
+ * Example:
+ *
+ *	{
+ * 		Tango::AutoTangoMonitor synch(this);
+ *		att_temp_seq.set_value (temp_seq, 100);
+ * 		att_temp_seq.fire_archive_event ();
+ *	}
+ *
+ * @param except A pointer to a DevFailed exception to be thrown as archive event.
+ */	 	
+	void fire_change_event(DevFailed *except = NULL);
+
+/**
+ * Set a flag to indicate that the server fires change events manually, without
+ * the polling to be started for the attribute.
+ * If the detect parameter is set to true, the criteria specified for the change 
+ * event are verified and the event is only pushed if they are fullfilled.
+ * If detect is set to false the event is fired without any value checking!
+ *
+ * @param implemented True when the server fires change events manually. 
+ * @param detect Triggers the verification of the change event properties when set to true.
+ */
+	void set_change_event(bool implemented, bool detect = true) 
+			{ ext->change_event_implmented = implemented;
+			  ext->check_change_event_criteria = detect;
+			  if(detect==false){
+			  ext->prev_change_event.err=false;
+			  ext->prev_change_event.quality=Tango::ATTR_VALID;}}	
+/**
+ * Check if the change event is fired manually (without polling) for this attribute.
+ *
+ * @return A boolean set to true if a manual fire change event is implemented.
+ */			
+	bool is_change_event() {return ext->change_event_implmented;}
+/**
+ * Check if the change event criteria should be checked when firing
+ * the event manually.
+ *
+ * @return A boolean set to true if a change event criteria will be checked.
+ */			
+	bool is_check_change_criteria() {return ext->check_change_event_criteria;}
+	
+	
+/**
+ * Fire an archive event for the attribute value. The event is pushed to the notification
+ * daemon.
+ * The attibute data must be set with one of the Attribute::set_value or 
+ * Attribute::setvalue_date_quality methods before fireing the event.
+ * The event is triggered with or without the archive event criteria depending
+ * on the configuration choosen with set_archive_event().
+ * ATTENTION: The couple set_value() and fire_archive_event() needs to be protected
+ * against concurrent accesses to the same attribute. Such an access might happen during 
+ * a synchronous read or by a reading from the polling thread.
+ * Inside all methods reading or writing commands and attributes this protection is
+ * automatically done by the Tango serialisation monitor.
+ * When fireing archive events in your own code, you should use the push_archive_event
+ * methods of the DeviceImpl class or protect your code with the
+ * Tango::AutoTangoMonitor on your device.
+ * Example:
+ *
+ *	{
+ * 		Tango::AutoTangoMonitor synch(this);
+ *		att_temp_seq.set_value (temp_seq, 100);
+ * 		att_temp_seq.fire_archive_event ();
+ *	}
+ *
+ * @param except A pointer to a DevFailed exception to be thrown as archive event.
+ */	
+	void fire_archive_event(DevFailed *except = NULL);
+
+/**
+ * Set a flag to indicate that the server fires archive events manually, without
+ * the polling to be started for the attribute
+ * If the detect parameter is set to true, the criteria specified for the archive 
+ * event are verified and the event is only pushed if they are fullfilled.
+ * If detect is set to false the event is fired without any value checking!
+ *
+ * @param implemented True when the server fires archive events manually.
+ * @param detect Triggers the verification of the archive event properties when set to true.
+ */
+	void set_archive_event(bool implemented, bool detect = true) 
+			{ext->archive_event_implmented = implemented;
+			 ext->check_archive_event_criteria = detect;}	
+
+/**
+ * Check if the archive event is fired manually for this attribute.
+ *
+ * @return A boolean set to true if a manual fire archive event is implemented.
+ */			
+	bool is_archive_event() {return ext->archive_event_implmented;}	
+/**
+ * Check if the archive event criteria should be checked when firing
+ * the event manually.
+ *
+ * @return A boolean set to true if a archive event criteria will be checked.
+ */			
+	bool is_check_archive_criteria() {return ext->check_archive_event_criteria;}
+
+/**
+ * Fire a user event for the attribute value. The event is pushed to the notification
+ * daemon.
+ * The attibute data must be set with one of the Attribute::set_value or 
+ * Attribute::setvalue_date_quality methods before fireing the event.
+ * ATTENTION: The couple set_value() and fire_event() needs to be protected
+ * against concurrent accesses to the same attribute. Such an access might happen during 
+ * a synchronous read or by a reading from the polling thread.
+ * Inside all methods reading or writing commands and attributes this protection is
+ * automatically done by the Tango serialisation monitor.
+ * When fireing archive events in your own code, you should use the push_event
+ * methods of the DeviceImpl class or protect your code with the
+ * Tango::AutoTangoMonitor on your device.
+ * Example:
+ *
+ *	{
+ * 		Tango::AutoTangoMonitor synch(this);
+ *		att_temp_seq.set_value (temp_seq, 100);
+ * 		att_temp_seq.fire_event ();
+ *	}
+ *
+ * @param filt_names The filterable fields name
+ * @param filt_vals The filterable fields value (as double)
+ * @param except A pointer to a DevFailed exception to be thrown as archive event.
+ */	
+	void fire_event(vector<string> &filt_names,vector<double> &filt_vals,DevFailed *except = NULL);		
 //@}
 
 //
 // methods not usable for the external world
 //
-
 	virtual void set_rvalue() {};
 	void delete_seq();
+	bool check_scalar_wattribute();
 	void wanted_date(bool flag) {date = flag;}
 	Tango::TimeVal &get_when() {return when;}
 	void set_time();
@@ -1179,11 +1374,11 @@ public:
 	bool quality_event_subscribed() {if (ext->event_quality_subscription != 0)return true;else return false;}
 	bool user_event_subscribed() {if (ext->event_user_subscription != 0)return true;else return false;}
 
-	void set_change_event_sub() {ext->event_change_subscription=time(NULL);}
-	void set_periodic_event_sub() {ext->event_periodic_subscription=time(NULL);}
-	void set_archive_event_sub() {ext->event_archive_subscription=time(NULL);}
-	void set_quality_event_sub() {ext->event_quality_subscription=time(NULL);}
-	void set_user_event_sub() {ext->event_user_subscription=time(NULL);}		
+	void set_change_event_sub() {ext->event_change_subscription=(long)time(NULL);}
+	void set_periodic_event_sub() {ext->event_periodic_subscription=(long)time(NULL);}
+	void set_archive_event_sub() {ext->event_archive_subscription=(long)time(NULL);}
+	void set_quality_event_sub() {ext->event_quality_subscription=(long)time(NULL);}
+	void set_user_event_sub() {ext->event_user_subscription=(long)time(NULL);}		
 
 	long get_attr_idx() {return ext->idx_in_attr;}
 
