@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device.cpp,v 3.12 2005/06/29 08:30:53 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: device.cpp,v 3.23 2006/06/29 11:49:45 jensmeyer Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -15,9 +15,50 @@ static const char *RcsId = "$Id: device.cpp,v 3.12 2005/06/29 08:30:53 taurel Ex
 //
 // author(s) :		A.Gotz + E.Taurel
 //
-// $Revision: 3.12 $
+// $Revision: 3.23 $
 //
 // $Log: device.cpp,v $
+// Revision 3.23  2006/06/29 11:49:45  jensmeyer
+// Added init_attr_poll_ext_trig() and init_attr_poll_period() to initialise the attribute
+// polling from the code.
+//
+// Revision 3.22  2006/06/28 10:37:47  taurel
+// - Some optimization in the init_cmd_poll_period and init_cmd_poll_ext_trig methods.
+//
+// Revision 3.21  2006/06/27 17:33:42  jensmeyer
+// Added init_cmd_poll_ext_trig() and init_cmd_poll_period() to write the polling configuration for commands to the database.
+// When specifying a polling of a command with Pogo now the configuration
+// get written into the database at startup time.
+//
+// Revision 3.20  2006/06/06 10:44:05  taurel
+// - Modified the DeviceImpl::push_event() method used to send user event.
+// It is now coherent with the other events type (change and archhive)
+//
+// Revision 3.19  2006/06/06 07:12:15  taurel
+// - Fix bug in push_event() method when the attribute is the device status
+//
+// Revision 3.18  2006/05/30 11:17:56  jensmeyer
+// Modifications on push_event functionality for MS Windows.
+// Mainly modifications for timing and time stamps.
+//
+// Revision 3.17  2006/05/23 12:05:56  jensmeyer
+// Added methods to push change and archive events.
+//
+// Revision 3.16  2006/04/27 13:47:11  taurel
+// - Merge the two DeviceImpl::push_event() methods in only one with only one method supporting also state or status attributes
+//
+// Revision 3.15  2006/04/13 13:31:44  taurel
+// Several changes:
+// - A new DeviceImpl::push_event() method to push USER event with State as data
+// - Fix bug in polling thread POLL_IUPD_PERIOD. The new polling period was applied only after the following poll still done with the old polling period
+// - It is now possible to update polling period of polled objects from a polled attribute or command
+//
+// Revision 3.14  2006/03/20 15:06:44  jensmeyer
+// Modified get_polled_obj_by_type_name() for porting to VC8
+//
+// Revision 3.13  2006/01/20 08:22:29  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 3.12  2005/06/29 08:30:53  taurel
 // - Last commit before release 5.2 ?
 //
@@ -330,6 +371,12 @@ DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,const char *d_name,
 #ifdef TANGO_HAS_LOG4TANGO
 	init_logger();
 #endif
+
+// 
+// write the polling
+//
+	init_cmd_poll_period();
+	init_attr_poll_period();
 }
 
 DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,string &d_name)
@@ -398,7 +445,12 @@ DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,string &d_name)
 #ifdef TANGO_HAS_LOG4TANGO
 	init_logger();
 #endif
-	
+
+// 
+// write the polling
+//
+	init_cmd_poll_period();	
+	init_attr_poll_period();
 }
 
 DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,string &d_name,string &description)
@@ -467,6 +519,12 @@ DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,string &d_name,string &description)
 #ifdef TANGO_HAS_LOG4TANGO
 	init_logger();
 #endif
+
+// 
+// write the polling
+//
+	init_cmd_poll_period();
+	init_attr_poll_period();
 }
 
 DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,string &d_name,string &de,
@@ -532,7 +590,12 @@ DeviceImpl::DeviceImpl(DeviceClass *cl_ptr,string &d_name,string &de,
 #ifdef TANGO_HAS_LOG4TANGO
 	init_logger();
 #endif
-	
+
+// 
+// write the polling
+//
+	init_cmd_poll_period();
+	init_attr_poll_period();	
 }
 
 //+-------------------------------------------------------------------------
@@ -1013,7 +1076,10 @@ vector<PollObj *>::iterator DeviceImpl::get_polled_obj_by_type_name(
 // Only to make compiler quiet. Should never pass here
 //
 
+// exclude the return value for VC8+
+#if !defined(WIN32) || (defined(_MSC_VER) && _MSC_VER < 1400)
 	return (vector<PollObj *>::iterator)NULL;
+#endif
 }
 
 //+-------------------------------------------------------------------------
@@ -1417,7 +1483,6 @@ throw (Tango::DevFailed, CORBA::SystemException)
 //
 
 	CORBA::Any *out_any;
-	transform(command.begin(),command.end(),command.begin(),::tolower);
 		
 	out_any = device_class->command_handler(this,command,in_any);
 		
@@ -2978,129 +3043,6 @@ void DeviceImpl::remove_attribute(string &rem_attr_name, bool free_it)
 	
 }
 
-//+-------------------------------------------------------------------------
-//
-// method :		DeviceImpl::push_event
-// 
-// description :	Push a user event to the Notification service
-//
-// argument: in :	- new_attr: The new attribute to be added.
-//
-//--------------------------------------------------------------------------
-
-void DeviceImpl::push_event(vector<string> &filterable_names,
-			    vector<double> &filterable_value,
-			    Tango::Attribute &att,
-			    Tango::DevErrorList *err)
-{
-	EventSupplier *event_supplier;
-	event_supplier = Util::instance()->get_event_supplier();
-	if (event_supplier != NULL)
-	{
-		Tango::DevFailed *ex;
-
-//
-// Build a pseudo exception
-//
-		if (err != NULL)
-			ex = new Tango::DevFailed(*err);
-		else
-			ex = NULL;
-
-//
-// Build the attribute value object
-//
-
-		Tango::AttributeValue att_val;
-		Tango::AttrQuality qual = att.get_quality();
-		if (qual != Tango::ATTR_INVALID)
-		{
-			Tango::AttrWriteType w_type = att.get_writable();
-			if ((w_type == Tango::READ) ||
-			    (w_type == Tango::READ_WRITE) ||
-			    (w_type == Tango::READ_WITH_WRITE))
-			{
-				if ((w_type == Tango::READ_WRITE) ||
-				    (w_type == Tango::READ_WITH_WRITE))
-					dev_attr->add_write_value(att);
-					
-				if ((att.is_alarmed().any() == true) && (qual != Tango::ATTR_INVALID))
-					att.check_alarm();
-			}
-
-			CORBA::Any &a = att_val.value;
-			switch (att.get_data_type())
-			{
-			case Tango::DEV_SHORT : 		
-				a <<= att.get_short_value();
-				break;
-
-			case Tango::DEV_LONG :
-				a <<= att.get_long_value();
-				break;
-
-			case Tango::DEV_DOUBLE :
-				a <<= att.get_double_value();
-				break;
-
-			case Tango::DEV_STRING :
-				a <<= att.get_string_value();
-				break;
-				
-			case Tango::DEV_FLOAT : 		
-				a <<= att.get_float_value();
-				break;
-
-			case Tango::DEV_BOOLEAN :
-				a <<= att.get_boolean_value();
-				break;
-
-			case Tango::DEV_USHORT :
-				a <<= att.get_ushort_value();
-				break;
-
-			case Tango::DEV_UCHAR :
-				a <<= att.get_uchar_value();
-				break;
-			}
-		}
-
-		att_val.time = att.get_when();
-		att_val.quality = qual;
-		att_val.name = CORBA::string_dup(att.get_name().c_str());
-		att_val.dim_x = att.get_x();
-		att_val.dim_y = att.get_y();
-		
-//
-// Push the event
-//
-
-		try
-		{
-			vector<string> filterable_names_lg;
-			vector<long> filterable_data_lg;
-					
-			event_supplier->push_event(this,
-					   "user_event",
-					   filterable_names,
-					   filterable_value,
-					   filterable_names_lg,
-					   filterable_data_lg,
-					   att_val,
-					   att.get_name(),
-					   ex);
-		}
-		catch (...)
-		{
-			delete ex;
-			throw;
-		}
-					   
-		delete ex;
-							       
-	}
-}
-
 
 //+-------------------------------------------------------------------------
 //
@@ -3183,7 +3125,8 @@ DeviceImplExt::DeviceImplExt()
     logger(0),saved_log_level(log4tango::Level::WARN),
     rft(Tango::kDefaultRollingThreshold),
     store_in_bb(true),poll_mon("cache"),
-    att_conf_mon("att_config"),state_from_read(false)
+    att_conf_mon("att_config"),state_from_read(false),
+    py_device(false)
 {
 }
 #endif
@@ -3216,4 +3159,330 @@ DeviceImplExt::~DeviceImplExt()
 }
 
 
+//+-------------------------------------------------------------------------
+//
+// method : 		Attribute::init_cmd_poll_ext_trig
+// 
+// description : 	Write the command name to the list of polled commands in the database.
+//			The polling period is set to 0 to indicate that the polling buffer
+//			is filled externally from the device server code.
+//
+//--------------------------------------------------------------------------
+
+void DeviceImpl::init_cmd_poll_ext_trig(string cmd_name)
+{
+	string  cmd_lowercase(cmd_name);
+   	transform(cmd_lowercase.begin(),cmd_lowercase.end(),cmd_lowercase.begin(),::tolower);				
+
+	// never do the for the state or status commands, they are
+	// handled as attributes!
+	if ( cmd_name == "state" || cmd_name == "status" )
+	{
+		TangoSys_OMemStream o;
+		
+		o << "State and status are handled as attributes for the polling" << ends;
+		Except::throw_exception((const char *)"API_CommandNotFound",
+				      o.str(),
+				      (const char *)"DeviceImpl::init_poll_ext_trig");		
+	}
+	
+	// check whether the command exists for the device and can be polled
+	check_command_exists (cmd_lowercase);
+	
+	
+	// check wether the database is used
+	Tango::Util *tg = Tango::Util::instance();	
+	if ( tg->_UseDb == true )
+	{
+		vector <string> &poll_list = get_polled_cmd();
+		Tango::DbData poll_data;
+		bool found = false;
+		
+		poll_data.push_back(Tango::DbDatum("polled_cmd"));
+	
+		// read the polling configuration from the database
+		//		tg->get_database()->get_device_property(device_name, poll_data);
+		
+		//		if (poll_data[0].is_empty()==false) 
+		if (poll_list.empty() == false)
+		{			
+			//poll_data[0] >> poll_list;
+			
+			// search the attribute in the list of polled attributes
+			for (unsigned int i = 0;i < poll_list.size();i = i+2)
+			{
+            //      Convert to lower case before comparison
+          			string  name_lowercase(poll_list[i]);
+            			transform(name_lowercase.begin(),name_lowercase.end(),name_lowercase.begin(),::tolower);
+				
+				if ( name_lowercase ==  cmd_lowercase)
+				{		
+					poll_list[i+1] = "0";
+					found = true;				
+				}
+			}
+		}
+			
+		if ( found == false )
+		{	
+			poll_list.push_back (cmd_lowercase);
+			poll_list.push_back ("0");		
+		}
+	
+		poll_data[0] << poll_list;	
+		tg->get_database()->put_device_property(device_name, poll_data);
+	}
+}
+
+//+-------------------------------------------------------------------------
+//
+// method : 		Attribute::init_cmd_poll_period
+// 
+// description : 	Checks the specified polling period for all commands of the device.
+//			If a polling period is specified for a command the
+//			command name and the period are written to the list of polled 
+//			commands in the database.
+//			This happens only if the command is not yet in the list of polled commands.
+//
+//--------------------------------------------------------------------------
+
+void DeviceImpl::init_cmd_poll_period()
+{
+	// check wether the database is used
+	Tango::Util *tg = Tango::Util::instance();	
+	if ( tg->_UseDb == true )
+	{
+		vector <string> &poll_list = get_polled_cmd();
+		Tango::DbData poll_data;
+		
+		poll_data.push_back(Tango::DbDatum("polled_cmd"));
+	
+		// read the polling configuration from the database
+		//		tg->get_database()->get_device_property(device_name, poll_data);
+		//		if (poll_data[0].is_empty()==false) 
+		//		{
+		//			poll_data[0] >> poll_list;
+		//		}
+			
+		// get the command list
+		vector<Command *> &cmd_list = device_class->get_command_list(); 
+
+		// loop over the command list
+		unsigned long added_cmd = 0;
+		unsigned long i;
+		for (i = 0;i < cmd_list.size();i++)
+		{
+			long poll_period;
+			poll_period = cmd_list[i]->get_polling_period();
+
+			//check the validity of the polling period.
+			// must be longer than 20ms
+			if ( poll_period < MIN_POLL_PERIOD )
+			{
+				continue;
+			}
+
+			// never do the for the state or status commands, they are
+			// handled as attributes!
+			string cmd_name = cmd_list[i]->get_lower_name();
+			if ( cmd_name == "state" || cmd_name == "status" )
+			{
+				continue;		
+			}
+
+			// Can only handle commands without input argument
+			if (cmd_list[i]->get_in_type() != Tango::DEV_VOID)
+			{	
+				continue;
+			}
+
+			// search the command in the list of polled commands
+			bool found = false;
+			for (unsigned int i = 0;i < poll_list.size();i = i+2)
+			{
+            //      Convert to lower case before comparison
+           			string  name_lowercase(poll_list[i]);
+            			transform(name_lowercase.begin(),name_lowercase.end(),name_lowercase.begin(),::tolower);
+
+				if ( name_lowercase == cmd_name)
+				{		
+					found = true;
+					break;				
+				}
+			}
+
+			if ( found == false )
+			{	
+				string period_str;
+				stringstream str;
+				str << poll_period << ends;
+				str >> period_str;
+
+				poll_list.push_back (cmd_name);
+				poll_list.push_back (period_str);
+				added_cmd++;
+			}
+		}
+		
+		// only write to the database when a polling need to be added
+		if ( added_cmd > 0 )
+		{			
+			poll_data[0] << poll_list;	
+			tg->get_database()->put_device_property(device_name, poll_data);
+		}
+	}
+}
+
+//+-------------------------------------------------------------------------
+//
+// method : 		Attribute::init_attr_poll_ext_trig
+// 
+// description : 	Write the attribute name to the list of polled attributes in the database.
+//			The polling period is set to 0 to indicate that the polling buffer
+//			is filled externally from the device server code.
+//
+//--------------------------------------------------------------------------
+
+void DeviceImpl::init_attr_poll_ext_trig(string attr_name)
+{
+	string  attr_lowercase(attr_name);
+   	transform(attr_lowercase.begin(),attr_lowercase.end(),attr_lowercase.begin(),::tolower);				
+	
+	// check whether the attribute exists for the device and can be polled
+	Attribute &req_attribute = dev_attr->get_attr_by_name (attr_lowercase.c_str());
+	
+	// check wether the database is used
+	Tango::Util *tg = Tango::Util::instance();	
+	if ( tg->_UseDb == true )
+	{
+		vector <string> &poll_list = get_polled_attr();
+		Tango::DbData poll_data;
+		bool found = false;
+		
+		poll_data.push_back(Tango::DbDatum("polled_attr"));
+	
+		// read the polling configuration from the database
+		//		tg->get_database()->get_device_property(device_name, poll_data);
+		
+		//		if (poll_data[0].is_empty()==false) 
+		if (poll_list.empty() == false)
+		{			
+			//poll_data[0] >> poll_list;
+			
+			// search the attribute in the list of polled attributes
+			for (unsigned int i = 0;i < poll_list.size();i = i+2)
+			{
+            //      Convert to lower case before comparison
+          			string  name_lowercase(poll_list[i]);
+            			transform(name_lowercase.begin(),name_lowercase.end(),name_lowercase.begin(),::tolower);
+				
+				if ( name_lowercase ==  attr_lowercase)
+				{		
+					poll_list[i+1] = "0";
+					found = true;				
+				}
+			}
+		}
+			
+		if ( found == false )
+		{	
+			poll_list.push_back (attr_lowercase);
+			poll_list.push_back ("0");	
+		}
+					
+		poll_data[0] << poll_list;	
+		tg->get_database()->put_device_property(device_name, poll_data);	
+	}
+}
+
+//+-------------------------------------------------------------------------
+//
+// method : 		Attribute::init_attr_poll_period
+// 
+// description : 	Checks the specified polling period for all attributes of the device.
+//			If a polling period is specified for an attribute the
+//			command name and the period are written to the list of polled 
+//			attributes in the database.
+//			This happens only if the attribute is not yet in the list of polled attributes.
+//
+//--------------------------------------------------------------------------
+
+void DeviceImpl::init_attr_poll_period()
+{
+	// check wether the database is used
+	Tango::Util *tg = Tango::Util::instance();	
+	if ( tg->_UseDb == true )
+	{
+		vector <string> &poll_list = get_polled_attr();
+		Tango::DbData poll_data;
+		
+		poll_data.push_back(Tango::DbDatum("polled_attr"));
+	
+		// read the polling configuration from the database
+		//		tg->get_database()->get_device_property(device_name, poll_data);
+		//		if (poll_data[0].is_empty()==false) 
+		//		{
+		//			poll_data[0] >> poll_list;
+		//		}
+			
+		// get the multi attribute object
+		vector<Attribute *> &attr_list = dev_attr->get_attribute_list(); 
+
+		// loop over the command list
+		unsigned long added_attr = 0;
+		unsigned long i;
+		for (i = 0;i < attr_list.size();i++)
+		{
+			long poll_period;
+			poll_period = attr_list[i]->get_polling_period();
+
+			//check the validity of the polling period.
+			// must be longer than 20ms
+			if ( poll_period < MIN_POLL_PERIOD )
+			{
+				continue;
+			}
+
+			string attr_name = attr_list[i]->get_name_lower();
+
+			// search the attribute in the list of polled attributes
+			bool found = false;
+			for (unsigned int i = 0;i < poll_list.size();i = i+2)
+			{
+            //      Convert to lower case before comparison
+           			string  name_lowercase(poll_list[i]);
+            			transform(name_lowercase.begin(),name_lowercase.end(),name_lowercase.begin(),::tolower);
+
+				if ( name_lowercase == attr_name)
+				{		
+					found = true;
+					break;				
+				}
+			}
+
+			if ( found == false )
+			{	
+				string period_str;
+				stringstream str;
+				str << poll_period << ends;
+				str >> period_str;
+
+				poll_list.push_back (attr_name);
+				poll_list.push_back (period_str);
+				added_attr++;
+			}
+		}
+		
+		// only write to the database when a polling need to be added
+		if ( added_attr > 0 )
+			{			
+			poll_data[0] << poll_list;	
+			tg->get_database()->put_device_property(device_name, poll_data);
+			}
+	}
+}
+
+
+
+
 } // End of Tango namespace
