@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_3.cpp,v 3.10 2005/05/09 15:34:10 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: device_3.cpp,v 3.18 2006/06/20 08:00:04 jensmeyer Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -15,9 +15,44 @@ static const char *RcsId = "$Id: device_3.cpp,v 3.10 2005/05/09 15:34:10 taurel
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.10 $
+// $Revision: 3.18 $
 //
 // $Log: device_3.cpp,v $
+// Revision 3.18  2006/06/20 08:00:04  jensmeyer
+// After changing the the init value of the attribute polling period to -1 insted of 0 I detected
+// a campatibility problem with the old versions. I changed back the init value to 0 and
+// solved the problem with new methods on the Attribute object.
+//
+// Revision 3.17  2006/06/14 11:21:35  jensmeyer
+// Modifications to make the set_polling_period() for attributes work
+// with a polling period set to 0.
+//
+// Revision 3.16  2006/05/30 11:17:56  jensmeyer
+// Modifications on push_event functionality for MS Windows.
+// Mainly modifications for timing and time stamps.
+//
+// Revision 3.15  2006/05/18 08:52:37  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
+// Revision 3.14  2006/04/28 13:17:20  taurel
+// - Attribute class: Add state and status attribute special case to the method Attribute_2_AttributeValue
+// - Device_3Impl: The monitor taken by the read_attribute_history_3 call was the Device Monitor. Replaced it by the polling list monitor
+//
+// Revision 3.13  2006/01/27 14:27:09  taurel
+// - Fix a severe incompatibility problem introduced by all the modifs done for
+// PY DS
+// - Duplicate some EventSupplier class methods (instead of using template) in order to be able to generate Tango shared library on Suse 9.3
+//
+// Revision 3.12  2006/01/20 08:25:54  taurel
+// - Added necessary changes to support Device server written in Python
+//
+// Revision 3.11  2005/11/02 12:54:13  jensmeyer
+// Avoid calling status2attr() during attribute reading when an error was detected.
+// Avoids segmentation fault in status2attr().
+//
 // Revision 3.10  2005/05/09 15:34:10  taurel
 // - Fix deadlock when read_attribute_no_cache try to start attribute polling
 // - Signal management for Sun
@@ -168,19 +203,23 @@ namespace Tango
 //--------------------------------------------------------------------------
 
 Device_3Impl::Device_3Impl(DeviceClass *device_class,string &dev_name):
-Device_2Impl(device_class,dev_name),ext_3(NULL)
+Device_2Impl(device_class,dev_name)
 {
 	ext->idl_version = 3;
 	add_state_status_attrs();
+	
+	ext_3 = new Device_3ImplExt();
 }
 		
 Device_3Impl::Device_3Impl(DeviceClass *device_class,
 			   string &dev_name,
 			   string &desc):
-Device_2Impl(device_class,dev_name,desc),ext_3(NULL)
+Device_2Impl(device_class,dev_name,desc)
 {
 	ext->idl_version = 3;
 	add_state_status_attrs();
+
+	ext_3 = new Device_3ImplExt();
 }
 		
 Device_3Impl::Device_3Impl(DeviceClass *device_class,
@@ -190,6 +229,8 @@ Device_2Impl(device_class,dev_name,desc,dev_state,dev_status),ext_3(NULL)
 {
 	ext->idl_version = 3;
 	add_state_status_attrs();
+
+	ext_3 = new Device_3ImplExt();
 } 
 		   
 Device_3Impl::Device_3Impl(DeviceClass *device_class,
@@ -197,10 +238,12 @@ Device_3Impl::Device_3Impl(DeviceClass *device_class,
 			   const char *desc,
 	           	   Tango::DevState dev_state,
 	           	   const char *dev_status):
-Device_2Impl(device_class,dev_name,desc,dev_state,dev_status),ext_3(NULL)
+Device_2Impl(device_class,dev_name,desc,dev_state,dev_status)
 {
 	ext->idl_version = 3;
 	add_state_status_attrs();
+
+	ext_3 = new Device_3ImplExt();
 }
 
 
@@ -555,6 +598,7 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 				}
 				attr_vect[att.get_attr_idx()]->read(this,att);
 //				read_attr(dev_attr->get_attr_by_ind(wanted_attr[i].idx_in_multi_attr));
+
 			}
 			catch (Tango::DevFailed &e)
 			{
@@ -684,7 +728,8 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 		
 		if ((status_wanted == true) && (status_idx == i))
 		{
-			status2attr(d_status,(*back)[index]);
+			if ((*back)[index].err_list.length() == 0)
+				status2attr(d_status,(*back)[index]);
 			continue;
 		}
 		
@@ -1138,7 +1183,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 			struct _timeb now_win;
 			_ftime(&now_win);
 			now.tv_sec = (unsigned long)now_win.time;
-			now.tv_usec = (long)now_win.millitm;
+			now.tv_usec = (long)now_win.millitm * 1000;
 #else
 			gettimeofday(&now,NULL);
 #endif
@@ -1459,7 +1504,15 @@ throw (Tango::MultiDevFailed, Tango::DevFailed, CORBA::SystemException)
 			errs[nb_failed - 1].index_in_call = ctr->idx_in_names;
 			errs[nb_failed - 1].err_list = e.errors;
 			ctr = updated_attr.erase(ctr);
-			--ctr;
+			if (ctr >= updated_attr.end())
+				break;
+			else
+			{
+				if (ctr == updated_attr.begin())
+					break;
+				else
+					--ctr;
+			}
 		}
 	}
 	
@@ -1504,7 +1557,15 @@ throw (Tango::MultiDevFailed, Tango::DevFailed, CORBA::SystemException)
 				errs[nb_failed - 1].index_in_call = (*ite).idx_in_names;
 				errs[nb_failed - 1].err_list = e.errors;
 				ite = updated_attr.erase(ite);
-				--ite;		
+				if (ite >= updated_attr.end())
+					break;
+				else
+				{
+					if (ite == updated_attr.begin())
+						break;
+					else
+						--ite;
+				}		
 			}
 		}
 	}
@@ -1580,8 +1641,9 @@ Tango::DevAttrHistoryList_3 *Device_3Impl::read_attribute_history_3(const char*
 								    CORBA::Long n)
 throw(Tango::DevFailed, CORBA::SystemException)
 {
-	AutoTangoMonitor sync(this);
-
+	TangoMonitor &mon = get_poll_monitor();
+	AutoTangoMonitor sync(&mon);
+	
 	cout4 << "Device_3Impl::read_attribute_history_3 arrived" << endl;
 
 //
