@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: deviceclass.cpp,v 1.3 2005/06/29 08:31:16 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: deviceclass.cpp,v 1.14 2006/06/21 14:52:19 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,51 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.3 2005/06/29 08:31:16 taure
 //
 // author(s) :		E.Taurel
 //
-// $Revision: 1.3 $
+// $Revision: 1.14 $
 //
 // $Log: deviceclass.cpp,v $
+// Revision 1.14  2006/06/21 14:52:19  taurel
+// - Just fixing a logging print message
+//
+// Revision 1.13  2006/06/12 14:22:33  jensmeyer
+// Corrected error handling for during initialisation of memorised attributes.
+//
+// Revision 1.12  2006/05/18 08:52:37  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
+// Revision 1.11  2006/03/09 08:18:06  taurel
+// - Change the order of device server destruction. First devices then class.
+// It was the contrary....
+//
+// Revision 1.10  2006/02/20 08:51:48  taurel
+// - A small edit mistake in delete_dev method...
+//
+// Revision 1.9  2006/02/20 08:30:00  taurel
+// - Add a new method in DeviceClass to delete a device at run-time
+//
+// Revision 1.8  2006/02/17 16:53:04  jensmeyer
+// Corrections when porting to VC7 under windows
+//
+// Revision 1.7  2006/01/27 14:27:10  taurel
+// - Fix a severe incompatibility problem introduced by all the modifs done for
+// PY DS
+// - Duplicate some EventSupplier class methods (instead of using template) in order to be able to generate Tango shared library on Suse 9.3
+//
+// Revision 1.6  2006/01/20 15:30:13  taurel
+// - Fixes in the default command management
+//
+// Revision 1.5  2006/01/20 08:30:05  taurel
+// - Added necessary changes to support Device server written in Python
+//
+// Revision 1.4  2005/11/07 12:15:00  jensmeyer
+// Corrected initialisation of memorised attributes. No longer exits when receiving an
+// exception during initialisation. Also added an option to only initialise the attribute setpoint and not writing to the attribute.
+// set_memorized_init(false) -> only applies setpoint.
+// set_memorized_init(true) -> also writes setpoint value to the attribute.
+//
 // Revision 1.3  2005/06/29 08:31:16  taurel
 // - Last commit before release 5.2 ?
 //
@@ -55,6 +97,7 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.3 2005/06/29 08:31:16 taure
 #include <logging.h>
 #endif
 
+extern omni_thread::key_t key_py_data;
 namespace Tango
 {
 
@@ -328,84 +371,128 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 					TangoSys_MemStream str;
 					str << mem_value << ends;
 					
-					switch (att.get_data_type())
+					try
 					{
-					case Tango::DEV_SHORT:
-						if (!(str >> sh))
-							throw_mem_value(device_list[i],att);
-						sh_seq[0] = sh;
-						att_val[nb_wr - 1].value <<= sh_seq;
-						break;
-			
-					case Tango::DEV_LONG:
-						if (!(str >> lg))
-							throw_mem_value(device_list[i],att);
-						lg_seq[0] = lg;
-						att_val[nb_wr - 1].value <<= lg_seq;
-						break;
-			
-					case Tango::DEV_DOUBLE:
-						if (!(str >> db))
-							throw_mem_value(device_list[i],att);
-						db_seq[0] = db;
-						att_val[nb_wr - 1].value <<= db_seq;
-						break;
+						switch (att.get_data_type())
+						{
+						case Tango::DEV_SHORT:
+							if (!(str >> sh))
+								throw_mem_value(device_list[i],att);
+							att.set_write_value(sh);
+							
+							sh_seq[0] = sh;
+							att_val[nb_wr - 1].value <<= sh_seq;
+							break;
+
+						case Tango::DEV_LONG:
+							if (!(str >> lg))
+								throw_mem_value(device_list[i],att);
+							att.set_write_value(lg);	
+								
+							lg_seq[0] = lg;
+							att_val[nb_wr - 1].value <<= lg_seq;
+							break;
+
+						case Tango::DEV_DOUBLE:
+							if (!(str >> db))
+								throw_mem_value(device_list[i],att);
+							att.set_write_value(db);
+							
+							db_seq[0] = db;
+							att_val[nb_wr - 1].value <<= db_seq;
+							break;
+
+						case Tango::DEV_STRING:
+							att.set_write_value(mem_value);
+							
+							str_seq[0] = CORBA::string_dup(mem_value.c_str());
+							att_val[nb_wr - 1].value <<= str_seq;
+							break;
+
+						case Tango::DEV_FLOAT:
+							if (!(str >> fl))
+								throw_mem_value(device_list[i],att);
+							att.set_write_value(fl);	
+								
+							fl_seq[0] = fl;
+							att_val[nb_wr - 1].value <<= fl_seq;
+							break;
+
+						case Tango::DEV_BOOLEAN:
+	#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+							if (!(str >> boolalpha >> boo))
+								throw_mem_value(device_list[i],att);
+	#else
+							transform(mem_value.begin(),
+								  mem_value.end(),
+								  mem_value.begin(),
+								  ::tolower);
+							if (mem_value == "true")
+								boo = true;
+							else if (mem_value == "false")
+								boo = false;
+							else
+								throw_mem_value(device_list[i],att);
+	#endif
+							att.set_write_value(boo);
+							
+							boo_seq[0] = boo;
+							att_val[nb_wr - 1].value <<= boo_seq;
+							break;
+
+						case Tango::DEV_USHORT:
+							if (!(str >> ush))
+								throw_mem_value(device_list[i],att);
+							att.set_write_value(ush);	
+								
+							ush_seq[0] = ush;
+							att_val[nb_wr - 1].value <<= ush_seq;
+							break;
+
+						case Tango::DEV_UCHAR:
+							if (!(str >> uch))
+								throw_mem_value(device_list[i],att);
+							att.set_write_value(uch);
+								
+							uch_seq[0] = uch;
+							att_val[nb_wr - 1].value <<= uch_seq;
+							break;
+						}
 						
-					case Tango::DEV_STRING:
-						str_seq[0] = CORBA::string_dup(mem_value.c_str());
-						att_val[nb_wr - 1].value <<= str_seq;
-						break;
-						
-					case Tango::DEV_FLOAT:
-						if (!(str >> fl))
-							throw_mem_value(device_list[i],att);
-						fl_seq[0] = fl;
-						att_val[nb_wr - 1].value <<= fl_seq;
-						break;
-			
-					case Tango::DEV_BOOLEAN:
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
-						if (!(str >> boolalpha >> boo))
-							throw_mem_value(device_list[i],att);
-#else
-						transform(mem_value.begin(),
-							  mem_value.end(),
-							  mem_value.begin(),
-							  ::tolower);
-						if (mem_value == "true")
-							boo = true;
-						else if (mem_value == "false")
-							boo = false;
-						else
-							throw_mem_value(device_list[i],att);
-#endif
-						boo_seq[0] = boo;
-						att_val[nb_wr - 1].value <<= boo_seq;
-						break;
-			
-					case Tango::DEV_USHORT:
-						if (!(str >> ush))
-							throw_mem_value(device_list[i],att);
-						ush_seq[0] = ush;
-						att_val[nb_wr - 1].value <<= ush_seq;
-						break;
-						
-					case Tango::DEV_UCHAR:
-						if (!(str >> uch))
-							throw_mem_value(device_list[i],att);
-						uch_seq[0] = uch;
-						att_val[nb_wr - 1].value <<= uch_seq;
-						break;
+					// Check the initialisation flag for memorized attributes.
+					// The the flag is false, do not add the element to the att_val
+					// vector. This avoids a call to write the memorozied value to the attribute.
+					
+					if ( att.is_memorized_init() == false )
+						{
+						nb_wr--;
+						att_val.length(nb_wr);
+						// reset memorized flag
+						att.set_memorized(true);
+						}
+					else
+						{
+						//
+						// Init the AttributeValue structure
+						//
+					
+						att_val[nb_wr - 1].name = CORBA::string_dup(att.get_name().c_str());
+						att_val[nb_wr - 1].dim_x = 1;
+						att_val[nb_wr - 1].dim_y = 0;						
+						att_val[nb_wr - 1].quality = Tango::ATTR_VALID;
+						}
 					}
-
-//
-// Init the AttributeValue structure
-//
-
-					att_val[nb_wr - 1].name = CORBA::string_dup(att.get_name().c_str());
-					att_val[nb_wr - 1].quality = Tango::ATTR_VALID;
-					att_val[nb_wr - 1].dim_x = 1;
-					att_val[nb_wr - 1].dim_y = 0;
+						
+					catch (Tango::DevFailed &e)
+					{
+						cout3 << "Cannot configure setpoint value for memorized attribute " << att.get_name() << endl;
+						Tango::Except::print_exception(e);
+						
+						nb_wr--;
+						att_val.length(nb_wr);
+						// reset memorized flag
+						att.set_memorized(true);
+					}						
 				}
 			}
 		}
@@ -416,40 +503,49 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 		if (nb_wr != 0)
 		{
 		
-//
-// Write attribute values. Re-throw exception if any but reset memorized flags before the
-// re-throw.
-//
+		//
+		// Write attribute values. Re-throw exception if any but reset memorized flags before the
+		// re-throw.
+		//
 			
 			try
 			{
 				cout4 << "Writing data for " << att_val.length() << " attribute(s) for device " << device_list[i]->get_name() << endl;
 				(static_cast<Device_3Impl *>(device_list[i]))->write_attributes_3(att_val);
 			}
-			catch (Tango::DevFailed)
+			catch (Tango::DevFailed &e)
 			{
-				for (unsigned long k = 0;k < att_val.length();k++)
-				{
-					WAttribute &att = device_list[i]->get_device_attr()->get_w_attr_by_name(att_val[k].name.in());
-					att.set_memorized(true);
-				}
-				throw;
+				cout3 << "Cannot write setpoint(s) value for memorized attribute(s) " << device_list[i]->get_name() << endl;
+				Tango::Except::print_exception(e);
+
+				//for (unsigned long k = 0;k < att_val.length();k++)
+				//{
+				//	WAttribute &att = device_list[i]->get_device_attr()->get_w_attr_by_name(att_val[k].name.in());
+				//	att.set_memorized(true);
+				//}
+				//throw;
 			}
 			catch (Tango::MultiDevFailed &e)
 			{
-				for (unsigned long k = 0;k < att_val.length();k++)
-				{
-					WAttribute &att = device_list[i]->get_device_attr()->get_w_attr_by_name(att_val[k].name.in());
-					att.set_memorized(true);
-				}
-				throw Tango::NamedDevFailedList(e,device_list[i]->get_name(),
-					       			(const char *)"DeviceClass::set_memorized_values()",
+				cout3 << "Cannot write setpoint(s) value for memorized attribute(s) " << device_list[i]->get_name() << endl;
+				
+				Tango::NamedDevFailedList e_list (e, device_list[i]->get_name(), (const char *)"DeviceClass::set_memorized_values()",
 					       			(const char *)"API_AttributeFailed");
+				Tango::Except::print_exception(e_list);
+
+				//for (unsigned long k = 0;k < att_val.length();k++)
+				//{
+				//	WAttribute &att = device_list[i]->get_device_attr()->get_w_attr_by_name(att_val[k].name.in());
+				//	att.set_memorized(true);
+				//}
+				//throw Tango::NamedDevFailedList(e,device_list[i]->get_name(),
+				//	       			(const char *)"DeviceClass::set_memorized_values()",
+				//	       			(const char *)"API_AttributeFailed");
 			}
 						
-//
-// Reset memorized flags
-//
+			//
+			// Reset memorized flags
+			//
 
 			for (unsigned long k = 0;k < att_val.length();k++)
 			{
@@ -509,55 +605,19 @@ DeviceClass::~DeviceClass()
 // Destroy the device list
 //
 
-	Tango::Util *tg = Tango::Util::instance();
-	PortableServer::POA_ptr r_poa = tg->get_poa();
 	unsigned long i;
-	
-	for (i = 0;i < device_list.size();i++)
+	if (device_list.size() != 0)
 	{
-	
-//
-// If the polling thread is alive and if device is polled,
-// ask polling thread to stop polling
-//
+		Tango::Util *tg = Tango::Util::instance();
+		PortableServer::POA_ptr r_poa = tg->get_poa();
 
-		if ((tg->get_polling_thread_object() != NULL) &&
-		    (device_list[i]->is_polled() == true))
-			device_list[i]->stop_polling();
-	
-//
-// Deactivate the CORBA object
-//
-
-		if (device_list[i]->get_exported_flag() == true)
-			r_poa->deactivate_object(device_list[i]->get_obj_id().in());
-			
-//
-// Unfortunately, the deactivate_object on the POA immediately returns even if there
-// is a running call (see Henning/Vinoski page 500). Therefore, try to get the
-// object monitor which will be free only when the last running call will finish
-// May be there is a more CORBA way to do this.
-// Need to investigate
-//
-	
-		try
-		{
-			device_list[i]->get_dev_monitor().get_monitor();	
-			device_list[i]->get_dev_monitor().rel_monitor();
+		for (i = 0;i < device_list.size();i++)
+		{	
+			delete_dev(i,tg,r_poa);		
 		}
-		catch (Tango::DevFailed &)
-		{
-		}
-		
-//
-// Remove the C++ servant
-//
-
-		device_list[i]->_remove_ref();
-		
+		device_list.clear();
+		CORBA::release(r_poa);
 	}
-	device_list.clear();
-	CORBA::release(r_poa);
 
 //
 // Destroy the command list
@@ -585,10 +645,104 @@ DeviceClass::~DeviceClass()
 
 	delete ext;
 	
-	cout4 << "Leaving DeviceClass destructor for class " << name << endl;
-	
+	cout4 << "Leaving DeviceClass destructor for class " << name << endl;	
 }
 
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DeviceClass::delete_dev()
+// 
+// description :	delete a device from the class device list
+//
+//-----------------------------------------------------------------------------
+
+
+void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_poa)
+{
+	cout4 << "Entering DeviceClas::delete_dev method for device with index " << idx << endl;
+		
+//
+// If the polling thread is alive and if device is polled,
+// ask polling thread to stop polling
+//
+
+	if ((tg->get_polling_thread_object() != NULL) && (device_list[idx]->is_polled() == true))
+		device_list[idx]->stop_polling();
+	
+//
+// Deactivate the CORBA object
+//
+
+	if (device_list[idx]->get_exported_flag() == true)
+		r_poa->deactivate_object(device_list[idx]->get_obj_id().in());
+			
+//
+// Unfortunately, the deactivate_object on the POA immediately returns even if there
+// is a running call (see Henning/Vinoski page 500). Therefore, try to get the
+// object monitor which will be free only when the last running call will finish
+// Two cases have to be taken into account :
+// 1 - The running commands : The get/rel on the monitor will make the thread
+//     waiting for them
+// 2- BUT, there are also (for very demanding clients) cases where commands are
+//    registered by the ORB but the monitor owner. For this case, sleep a while
+//    to give them time to execute. But sleep how long ?
+//    By default, I choose 200 mS
+// May be there is a more CORBA way to do this.
+// Need to investigate
+//
+	omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
+	PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;		
+
+#ifdef WIN32
+	try
+	{
+		lock_ptr->Release();
+
+		device_list[idx]->get_dev_monitor().get_monitor();	
+		device_list[idx]->get_dev_monitor().rel_monitor();
+		
+		Sleep(200);
+
+		lock_ptr->Get();
+	}
+#else
+	try
+	{
+		lock_ptr->Release();
+
+		device_list[idx]->get_dev_monitor().get_monitor();
+		device_list[idx]->get_dev_monitor().rel_monitor();
+
+		struct timespec wait_to_sleep,still_wait;
+		wait_to_sleep.tv_sec = 0;
+		wait_to_sleep.tv_nsec = 200000000;
+
+		nanosleep(&wait_to_sleep,&still_wait);
+
+		lock_ptr->Get();
+	}
+#endif /* WIN32 */
+	catch (Tango::DevFailed &)
+	{
+	}
+		
+//
+// Remove the C++ servant
+//
+
+	if (device_list[idx]->is_py_device() == false)
+		device_list[idx]->_remove_ref();
+	else
+	{
+		Device_3Impl *dev_3 = static_cast<Device_3Impl *>(device_list[idx]);
+		dev_3->delete_dev();
+	}
+	
+	cout4 << "Leaving DeviceClass delete_dev" << endl;	
+}
+
+
 //+-------------------------------------------------------------------------
 //
 // method :		DeviceClass::register_signal
@@ -824,14 +978,19 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 {	
 	CORBA::Any *ret = NULL;
 	vector<Command *>::iterator i_cmd;
+	
+	bool with_default_cmd = false;
+	string command_lower(command);
 
 	cout4 << "Entering DeviceClass::command_handler() method" << endl;
+
+	transform(command_lower.begin(),command_lower.end(),command_lower.begin(),::tolower);
 	
 	for (i_cmd = command_list.begin();i_cmd < command_list.end();++i_cmd)
 	{
-		if ((*i_cmd)->get_lower_name() == command)
+		if ((*i_cmd)->get_lower_name() == command_lower)
 		{
-		
+
 //
 // Call the always executed method
 //
@@ -839,9 +998,9 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 			device->always_executed_hook();
 
 //
-// Chaeck if command is allowed
+// Check if command is allowed
 //
-			
+
 			if ((*i_cmd)->is_allowed(device,in_any) == false)
 			{
 				TangoSys_OMemStream o;
@@ -859,21 +1018,64 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 			break;
 		}
 	}
-		
+
 	if (i_cmd == command_list.end())
 	{
+	
 		cout3 << "DeviceClass::command_handler(): command " << command << " not found" << endl;
+
+
+		Command *def_cmd = get_default_command();
+		if (def_cmd != NULL)
+		{
 		
-//		
-// throw an exception to client
+//
+// Set name in default command object
 //
 
-		TangoSys_OMemStream o;
+			def_cmd->set_name(command);
+					
+//
+// Call the always executed method
+//
+
+			device->always_executed_hook();
+
+//
+// Check if command is allowed
+//
+
+			if (def_cmd->is_allowed(device,in_any) == false)
+			{
+				TangoSys_OMemStream o;
+				o << "Command " << command << " not allowed when the device is in " << Tango::DevStateName[device->get_state()] << " state"  << ends;
+				Except::throw_exception((const char *)"API_CommandNotAllowed",
+						o.str(),
+						(const char *)"DeviceClass::command_handler");
+			}
+	
+//
+// Execute command
+//
+
+			ret = def_cmd->execute(device,in_any);
+
+		}
+		else
+		{
 		
-		o << "Command " << command << " not found" << ends;
-		Except::throw_exception((const char *)"API_CommandNotFound",
-				      o.str(),
-				      (const char *)"DeviceClass::command_handler");
+					
+//		
+// Throw an exception to client
+//
+
+			TangoSys_OMemStream o;
+
+			o << "Command " << command << " not found" << ends;
+			Except::throw_exception((const char *)"API_CommandNotFound",
+					      o.str(),
+					      (const char *)"DeviceClass::command_handler");
+		}
 	}
 	
 	cout4 << "Leaving DeviceClass::command_handler() method" << endl;	
