@@ -8,9 +8,18 @@
 //
 // author(s) :		E.Taurel
 //
-// $Revision: 1.4 $
+// $Revision: 1.7 $
 //
 // $Log: deviceclass.h,v $
+// Revision 1.7  2006/02/20 08:16:52  taurel
+// - Add a new method in DeviceClass to delete a device at run-time
+//
+// Revision 1.6  2006/01/20 15:30:13  taurel
+// - Fixes in the default command management
+//
+// Revision 1.5  2006/01/20 08:22:29  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 1.4  2005/06/29 08:31:16  taurel
 // - Last commit before release 5.2 ?
 //
@@ -45,6 +54,7 @@ class DeviceClass;
 class AutoTangoMonitor;
 class NoSyncModelTangoMonitor;
 class EventSupplier;
+class Util;
 
 
 //=============================================================================
@@ -62,7 +72,7 @@ class EventSupplier;
 class DeviceClassExt
 {
 public:
-	DeviceClassExt():only_one("class") {};
+	DeviceClassExt():only_one("class"),default_cmd(NULL),py_class(false) {};
 	
 	vector<string>		nodb_name_list;
 	
@@ -70,6 +80,10 @@ public:
 	
 	string			cvs_tag;
 	string			cvs_location;
+	
+	Command * 		default_cmd;
+	bool			py_class;
+
 };
 
 /**
@@ -78,7 +92,7 @@ public:
  * device class
  *
  * $Author: taurel $
- * $Revision: 1.4 $
+ * $Revision: 1.7 $
  */
  
 class DeviceClass
@@ -308,7 +322,15 @@ public:
 	
 	string &get_cvs_tag() {return ext->cvs_tag;}
 	string &get_cvs_location() {return ext->cvs_location;}
-				
+	
+	void add_device(DeviceImpl *dev) {device_list.push_back(dev);}
+	void delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_poa);
+
+	bool is_py_class() {return ext->py_class;}	
+	void set_py_class(bool py) {ext->py_class=py;}
+	virtual void delete_class() {}	
+
+					
 protected:
 /**@name Constructor
  Only one constructot for this class which is a singleton */
@@ -340,7 +362,21 @@ protected:
  * <b>DevFailed</b> exception specification
  */	
 	void export_device(DeviceImpl *dev,const char* corba_dev_name = "Unused");
-//@}
+	
+/**
+ * Set a Tango classs default command
+ *
+ * Define one command to be the Tango class default command
+ * The default command is the command which will be exceuted when
+ * an unknown command is sent to one of the Tango class device
+ * By default, there is no default class
+ *
+ * @param cmd The command object
+ */	
+
+	void set_default_command(Command *cmd) {ext->default_cmd = cmd;}
+//@}	
+	Command *get_default_command() {return ext->default_cmd;}
 
 
 /**@name Class data members */
