@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverpoll.cpp,v 3.8 2005/06/29 08:31:17 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverpoll.cpp,v 3.10 2006/04/13 13:31:44 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,18 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.8 2005/06/29 08:31:17 taure
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.10 $
 //
 // $Log: dserverpoll.cpp,v $
+// Revision 3.10  2006/04/13 13:31:44  taurel
+// Several changes:
+// - A new DeviceImpl::push_event() method to push USER event with State as data
+// - Fix bug in polling thread POLL_IUPD_PERIOD. The new polling period was applied only after the following poll still done with the old polling period
+// - It is now possible to update polling period of polled objects from a polled attribute or command
+//
+// Revision 3.9  2006/01/20 08:12:17  taurel
+// - No real change in this file. Simply changes in indentation
+//
 // Revision 3.8  2005/06/29 08:31:17  taurel
 // - Last commit before release 5.2 ?
 //
@@ -325,7 +334,7 @@ Tango::DevVarStringArray *DServer::dev_poll_status(string &dev_name)
 
 	Tango::Util *tg = Tango::Util::instance();
 	DeviceImpl *dev;
-	
+
 	dev = tg->get_device_by_name(dev_name);
 
 	vector<PollObj *> &poll_list = dev->get_poll_obj_list();
@@ -818,10 +827,11 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 //
 
 	long upd = (argin->lvalue)[0];
+	
 	if ((upd != 0) && (upd < MIN_POLL_PERIOD))
 	{
 		TangoSys_OMemStream o;
-		o << (argin->lvalue)[0] << " is below the min authorized period (100 mS)" << ends;
+		o << (argin->lvalue)[0] << " is below the min authorized period (" << MIN_POLL_PERIOD << " mS)" << ends;
 		Except::throw_exception((const char *)"API_NotSupported",o.str(),
 					(const char *)"DServer::add_obj_polling");
 	}
@@ -1133,7 +1143,20 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 		Except::throw_exception((const char *)"API_NotSupported",o.str(),
 					(const char *)"DServer::upd_obj_polling_period");
 	}*/
-			
+
+//
+// Check that the update period is not to small
+//
+
+	long upd = (argin->lvalue)[0];
+	if ((upd != 0) && (upd < MIN_POLL_PERIOD))
+	{
+		TangoSys_OMemStream o;
+		o << (argin->lvalue)[0] << " is below the min authorized period (" << MIN_POLL_PERIOD << " mS)" << ends;
+		Except::throw_exception((const char *)"API_NotSupported",o.str(),
+					(const char *)"DServer::add_obj_polling");
+	}
+				
 //
 // Update polling period
 //
@@ -1147,6 +1170,9 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 	TangoMonitor &mon = tg->get_poll_monitor();
 	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
 
+	omni_thread *th = omni_thread::self();
+	int th_id = th->id();
+	if (th_id != tg->get_polling_thread_id())		
 	{	
 		omni_mutex_lock sync(mon);
 		if (shared_cmd.cmd_pending == true)
@@ -1162,6 +1188,19 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 	
 		mon.signal();
 	}
+	else
+	{
+		shared_cmd.cmd_pending = true;
+		shared_cmd.cmd_code = POLL_UPD_PERIOD;
+		shared_cmd.dev = dev;
+		shared_cmd.name = obj_name;
+		shared_cmd.type = type;
+		shared_cmd.new_upd = (argin->lvalue)[0];
+		
+		PollThread *poll_th = tg->get_polling_thread_object();
+		poll_th->set_local_cmd(shared_cmd);
+		poll_th->execute_cmd();
+	}
 	
 	
 //
