@@ -11,9 +11,12 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.1 $
+// $Revision: 3.2 $
 //
 // $Log: dserversignal.h,v $
+// Revision 3.2  2006/01/20 08:22:29  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 3.1  2004/07/07 08:40:11  taurel
 //
 // - Fisrt commit after merge between Trunk and release 4 branch
@@ -198,10 +201,11 @@ public :
 	{
 		DServerSignal *ds;
 	public:
-		ThSig(DServerSignal *d):ds(d),my_pid(0) {}
+		ThSig(DServerSignal *d):ds(d),my_pid(0),th_data_created(false) {}
 		virtual ~ThSig() {}
 		
 		TangoSys_Pid my_pid;
+		bool th_data_created;
 #ifndef WIN32
 		pthread_t my_thread;
 #endif
