@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: event.cpp,v 1.10 2005/07/20 14:40:46 taurel Exp $";
+static const char *RcsId = "$Id: event.cpp,v 1.13 2006/06/20 15:17:55 jensmeyer Exp $";
 
 ////////////////////////////////////////////////////////////////////////////////
 ///
@@ -14,9 +14,22 @@ static const char *RcsId = "$Id: event.cpp,v 1.10 2005/07/20 14:40:46 taurel Exp
 ///
 ///		original : 7 April 2003
 ///
-///		$Revision: 1.10 $
+///		$Revision: 1.13 $
 ///
 ///		$Log: event.cpp,v $
+///		Revision 1.13  2006/06/20 15:17:55  jensmeyer
+///		Corrected bug in the event reconnection.
+///		The bug was introduced when implementing the host name check
+///		for the notifd.
+///		
+///		Revision 1.12  2006/06/14 11:24:44  jensmeyer
+///		Modifications to reconnect to the good notifyd when changing a device server from
+///		one host to another. For all subscribed events of the server another notifyd needs
+///		to be used.
+///		
+///		Revision 1.11  2005/12/16 17:36:24  jensmeyer
+///		Suppressed the subscription for quality change events, throwing an exception now.
+///		
 ///		Revision 1.10  2005/07/20 14:40:46  taurel
 ///		- Fix bug in polling thread out of sync error. Time threshold to generate
 ///		event heartbeat error changed from 10 to 11. Fix some comments
@@ -387,6 +400,7 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 
 	int channel_exported;
 	string channel_ior;
+	string hostname;
 
 	if (db != NULL)
 	{
@@ -409,9 +423,12 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 			      			(const char *)"EventConsumer::connect_event_channel");
 		}
 	
-        	received.inout() >>= dev_import_list;
-        	channel_ior = string((dev_import_list->svalue)[1]);
-        	channel_exported = dev_import_list->lvalue[0];
+     	received.inout() >>= dev_import_list;
+     	channel_ior = string((dev_import_list->svalue)[1]);
+     	channel_exported = dev_import_list->lvalue[0];
+			
+	  	// get the hostname where the notifyd should be running
+		hostname = string (dev_import_list->svalue[3]);
 	}
 	else
 	{
@@ -423,6 +440,10 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 			dd = adm.command_inout("QueryEventChannelIOR");
 			dd >> channel_ior;
 			channel_exported = true;
+			
+			// get the hostname where the notifyd should be running
+			DeviceInfo info = adm.info();
+			hostname = info.server_host;		
 		}
 		catch (...)
 		{
@@ -573,6 +594,7 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 		channel_map[channel_name].structuredProxyPushSupplier = structuredProxyPushSupplier;
 		channel_map[channel_name].last_heartbeat = time(NULL);
 		channel_map[channel_name].heartbeat_skipped = false;
+		channel_map[channel_name].notifyd_host = hostname;
 	}
 	else
 	{
@@ -582,6 +604,7 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 		new_event_channel_struct.heartbeat_skipped = false;
 		new_event_channel_struct.adm_device_proxy = NULL;
 		channel_map[channel_name] = new_event_channel_struct;
+		channel_map[channel_name].notifyd_host = hostname;
 	}
 
 //
@@ -1060,7 +1083,11 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 				    break;
 
 		case QUALITY_EVENT : event_name = "quality";
-				     break;
+					  EventSystemExcept::throw_exception((const char*)"API_InvalidArgs",
+                       	(const char*)"The quality change event does`nt exist any more. A change event is fired on a qaulity change!",
+                       	(const char*)"EventConsumer::subscribe_event()");
+				    
+					  break;
 
 		case PERIODIC_EVENT : event_name = "periodic";
 				      break;
@@ -1815,7 +1842,29 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 				bool notifd_failed = false;
 				try
 				{
-					CosNotifyChannelAdmin::EventChannelFactory_var ecf = ipos->second.eventChannel->MyFactory();
+// 
+//  Check if the device server is now running on a different host.
+//  In this case we have to reconnect to another notification daemon.
+//						 
+					DeviceInfo info;
+					try
+					{
+						info = ipos->second.adm_device_proxy->info();
+					}
+					catch (Tango::DevFailed &e)
+					{
+						// in case of failure, just stay connected to the actual notifd
+						info.server_host = ipos->second.notifyd_host;	
+					}
+					
+					//cout << "Old host = " << ipos->second.notifyd_host << endl;
+					//cout << "New host = " << info.server_host << endl;
+					if ( ipos->second.notifyd_host != info.server_host )
+						{
+						notifd_failed = true;
+						}
+					else
+						CosNotifyChannelAdmin::EventChannelFactory_var ecf = ipos->second.eventChannel->MyFactory();
 				}
 				catch (...)
 				{
