@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: eventcmds.cpp,v 1.4 2005/06/29 08:31:18 taurel Exp $";
+static const char *RcsId = "$Id: eventcmds.cpp,v 1.11 2006/06/20 08:00:04 jensmeyer Exp $";
 //+=============================================================================
 //
 // file :        eventcmds.cpp
@@ -8,11 +8,51 @@ static const char *RcsId = "$Id: eventcmds.cpp,v 1.4 2005/06/29 08:31:18 taurel
 //
 // project :     TANGO events
 //
-// $Author: taurel $
+// $Author: jensmeyer $
 //
-// $Revision: 1.4 $
+// $Revision: 1.11 $
 //
 // $Log: eventcmds.cpp,v $
+// Revision 1.11  2006/06/20 08:00:04  jensmeyer
+// After changing the the init value of the attribute polling period to -1 insted of 0 I detected
+// a campatibility problem with the old versions. I changed back the init value to 0 and
+// solved the problem with new methods on the Attribute object.
+//
+// Revision 1.10  2006/06/14 11:21:35  jensmeyer
+// Modifications to make the set_polling_period() for attributes work
+// with a polling period set to 0.
+//
+// Revision 1.9  2006/05/18 14:29:12  jensmeyer
+// Changed the handling of the event period for periodic and archiving events.
+// The event period is now stored as the configured value and no longer
+// multilplied by the factor DELTA_PERIODIC.
+// The calculation of the precision to fire periodic events has moved from
+// the attribbute to the push methods in the event supplier.
+//
+// For event periods < 5000 ms a precision of 2% was kept, but for
+// times < 5000 ms a fixed precision of 100 ms was added.
+// This avoids periodic events with a long event period to be fired to early.
+//
+// Revision 1.8  2006/04/28 13:17:20  taurel
+// - Attribute class: Add state and status attribute special case to the method Attribute_2_AttributeValue
+// - Device_3Impl: The monitor taken by the read_attribute_history_3 call was the Device Monitor. Replaced it by the polling list monitor
+//
+// Revision 1.7  2006/04/13 06:12:28  jensmeyer
+// Added methods to configure and fire manually change and archive events without
+// polling.
+//
+// Revision 1.6  2005/12/16 17:38:57  jensmeyer
+// Changed the error checking during an attribute subscription.
+// Also added an option to allow manual change events.
+//
+// Revision 1.5  2005/11/09 09:43:57  jensmeyer
+// Stopped automatic polling when subscribing for events!
+// Added exceptions when polling is not started for an attribute.
+//
+// Changed the default of the archiving period from 10 seconds to
+// no periodic event!
+// Added exception when trying to subscribe a not configured attribute.
+//
 // Revision 1.4  2005/06/29 08:31:18  taurel
 // - Last commit before release 5.2 ?
 //
@@ -232,130 +272,180 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 	int attr_ind = dev_impl->dev_attr->get_attr_ind_by_name(attr_name.c_str());
 	Attribute &attribute = dev_impl->dev_attr->get_attr_by_ind(attr_ind);
 
+	
 	if (action == "subscribe")
 	{
-		bool send_poll_cmd = true;
-       		if (event == "change")
-       		{
-			cout4 << "EventSubscriptionChangeCmd::execute(): update change subscription\n";
-
-//
-// Check if the attribute has some of the change properties defined
-//
-
-			if (attr_name_lower != "state")
-			{
-				if ((attribute.get_data_type() != Tango::DEV_STRING) &&
-			            (attribute.get_data_type() != Tango::DEV_BOOLEAN))
-				{
-					if ((attribute.ext->rel_change[0] == INT_MAX) &&
-					    (attribute.ext->rel_change[1] == INT_MAX) &&
-					    (attribute.ext->abs_change[0] == INT_MAX) &&
-					    (attribute.ext->abs_change[1] == INT_MAX))
-					{
-						TangoSys_OMemStream o;
-						o << "Event properties (abs_change or rel_change) for attribute ";
-						o << attr_name;
-						o << " are not set" << ends;
-
-						Except::throw_exception((const char *)"API_EventPropertiesNotSet",
-							o.str(),
-							(const char *)"EventSubscriptionChangeCmd::execute");
-					}
-				}
-			}
-
-       			attribute.ext->event_change_subscription = time(NULL);
-		}
- 
-       		else if (event == "quality")
-       		{
-			cout4 << "EventSubscriptionChangeCmd::execute(): update quality_change subscription\n";
-       			attribute.ext->event_quality_subscription = time(NULL);
-       		}
- 
-       		else if (event == "periodic")
-       		{
-			cout4 << "EventSubscriptionChangeCmd::execute(): update periodic subscription\n";
-       			attribute.ext->event_periodic_subscription = time(NULL);
-       		}                                                  
- 
-       		else if (event == "archive")
-       		{
-			cout4 << "EventSubscriptionChangeCmd::execute(): update archive subscription\n";
-       			attribute.ext->event_archive_subscription = time(NULL);
-       		}
-		
-		else if (event == "user_event")
+		if (event == "user_event")
 		{
 			cout4 << "EventSubscriptionChangeCmd::execute(): update user_event subscription\n";
        			attribute.ext->event_user_subscription = time(NULL);
-			send_poll_cmd = false;
-       		}		  
-		                                                
-//
-// If polling period is 0 then set a default polling period of 1 second
-// Update archive period to 10 polling period only if the property
-// "archive_period" is not defined and if attribute is already polled
-// 
-
-		if (send_poll_cmd == true)
-		{
-			if (attribute.ext->poll_period <= 0) 
-			{
-				attribute.ext->poll_period = 1000;
-			}
-			else
-			{
-				if (event == "archive")
-				{
-					if (attribute.ext->archive_period == 10 * (int)(1000.0 * DELTA_PERIODIC))
-						attribute.ext->archive_period = 10 * attribute.ext->poll_period;
-				}
-			}
-		
-//
-// Start polling for attribute in question. I suppose I should
-// check to see if the attribute is polled already. For the 
-// moment I will simply ignore the exception. Why not rather 
-// introduce a is_polled() method in each Attribute ?
-//
-// Use the add_obj_polling() admin device method whith no
-// db update. The polling will poll the attribute next time
-// the server is started if  there no more event client connected
-// to the attribute
-//
+      		}
+		else
+		{		
+			// If the polling is necessary to send events, check whether the polling is
+			// started for the requested attribute.
 			
-			DevVarLongStringArray *send = new DevVarLongStringArray();
-			try
+			if (attribute.is_polled() == false )			
 			{
-				Tango::Util *tg = Tango::Util::instance();
-				DServer *adm_dev = tg->get_dserver_device();
-
-				if (adm_dev->get_heartbeat_started() == false)
-				{				
-					adm_dev->add_event_heartbeat();
-					adm_dev->set_heartbeat_started(true);
+				TangoSys_OMemStream o;
+				o << "The polling (necessary to send events) for the attribute ";
+				o << attr_name;
+				o << " is not started" << ends;
+								
+				if ( event == "change")
+				{
+					if (attribute.is_change_event() == false)
+					{
+						Except::throw_exception((const char *)"API_AttributePollingNotStarted",
+									o.str(),
+									(const char *)"EventSubscriptionChangeCmd::execute");
+					}
+				}
+				else
+				{
+					if ( event == "archive")
+					{
+						if (attribute.is_archive_event() == false)
+						{
+							Except::throw_exception((const char *)"API_AttributePollingNotStarted",
+										o.str(),
+										(const char *)"EventSubscriptionChangeCmd::execute");
+						}
+					}
+					else
+					{					
+						Except::throw_exception((const char *)"API_AttributePollingNotStarted",
+									o.str(),
+									(const char *)"EventSubscriptionChangeCmd::execute");					
+					}
 				}
-
-				send->lvalue.length(1);
-				send->svalue.length(3);
-				send->svalue[0] = CORBA::string_dup(dev_name.c_str());
-				send->svalue[1] = CORBA::string_dup("attribute");
-
-				send->lvalue[0] = attribute.ext->poll_period;
-				send->svalue[2] = CORBA::string_dup(attr_name.c_str());
-
-				cout4 << "EventSubscriptionChangeCmd::execute(): start attribute " << attr_name << " polling with period " << attribute.ext->poll_period << endl;
-				adm_dev->add_obj_polling(send,false);
 			}
-			catch (...)
-			{
+			
+				
+       	if (event == "change")
+       	{
+				cout4 << "EventSubscriptionChangeCmd::execute(): update change subscription\n";
+
+				//
+				// Check if the attribute has some of the change properties defined
+				//
+
+				if (attr_name_lower != "state")
+				{
+					if ((attribute.get_data_type() != Tango::DEV_STRING) &&
+			            	    (attribute.get_data_type() != Tango::DEV_BOOLEAN))
+					{
+						if ( attribute.is_check_change_criteria() == true )
+						{
+							if ((attribute.ext->rel_change[0] == INT_MAX) &&
+					    		    (attribute.ext->rel_change[1] == INT_MAX) &&
+					    		    (attribute.ext->abs_change[0] == INT_MAX) &&
+					    		    (attribute.ext->abs_change[1] == INT_MAX))
+							{		
+								TangoSys_OMemStream o;
+								o << "Event properties (abs_change or rel_change) for attribute ";
+								o << attr_name;
+								o << " are not set" << ends;
+
+								Except::throw_exception((const char *)"API_EventPropertiesNotSet",
+																o.str(),
+																(const char *)"EventSubscriptionChangeCmd::execute");
+							}
+						}
+					}
+				}
+       				attribute.ext->event_change_subscription = time(NULL);
+			}
+ 
+      			else if (event == "quality")
+      			{
+				cout4 << "EventSubscriptionChangeCmd::execute(): update quality_change subscription\n";
+       				attribute.ext->event_quality_subscription = time(NULL);
+      			}
+ 
+      			else if (event == "periodic")
+      			{
+				cout4 << "EventSubscriptionChangeCmd::execute(): update periodic subscription\n";
+       				attribute.ext->event_periodic_subscription = time(NULL);
+      			}                                                  
+ 
+      			else if (event == "archive")
+      			{
+				//
+				// Check if the attribute has some of the archive properties defined
+				//
+
+				if (attr_name_lower != "state")
+				{
+					if ((attribute.get_data_type() != Tango::DEV_STRING) &&
+			                    (attribute.get_data_type() != Tango::DEV_BOOLEAN))
+					{
+						if ( attribute.is_check_archive_criteria() == true )
+						{
+							if ((attribute.ext->archive_abs_change[0] == INT_MAX) &&
+					      		    (attribute.ext->archive_abs_change[1] == INT_MAX) &&
+						   	    (attribute.ext->archive_period        == INT_MAX))
+							{
+								TangoSys_OMemStream o;
+								o << "Archive event properties (archive_abs_change or archive_rel_change or archive_period) for attribute ";
+								o << attr_name;
+								o << " are not set" << ends;
+
+								Except::throw_exception((const char *)"API_EventPropertiesNotSet",
+															 	o.str(),
+																(const char *)"EventSubscriptionChangeCmd::execute");
+							}
+						}
+					}
+				}		
+		
+				cout4 << "EventSubscriptionChangeCmd::execute(): update archive subscription\n";
+       				attribute.ext->event_archive_subscription = time(NULL);
+      			}				 			
+		}		  
+	
+		//
+		// Start polling for attribute in question. I suppose I should
+		// check to see if the attribute is polled already. For the 
+		// moment I will simply ignore the exception. Why not rather 
+		// introduce a is_polled() method in each Attribute ?
+		//
+		// Use the add_obj_polling() admin device method whith no
+		// db update. The polling will poll the attribute next time
+		// the server is started if  there no more event client connected
+		// to the attribute
+		//
+			
+		//DevVarLongStringArray *send = new DevVarLongStringArray();
+		try
+		{
+			Tango::Util *tg = Tango::Util::instance();
+			DServer *adm_dev = tg->get_dserver_device();
+
+			if (adm_dev->get_heartbeat_started() == false)
+			{				
+				adm_dev->add_event_heartbeat();
+				adm_dev->set_heartbeat_started(true);
+			}
+
+				//send->lvalue.length(1);
+				//send->svalue.length(3);
+				//send->svalue[0] = CORBA::string_dup(dev_name.c_str());
+				//send->svalue[1] = CORBA::string_dup("attribute");
+
+				//send->lvalue[0] = attribute.ext->poll_period;
+				//send->svalue[2] = CORBA::string_dup(attr_name.c_str());
+
+				//cout4 << "EventSubscriptionChangeCmd::execute(): start attribute " << attr_name << " polling with period " << attribute.ext->poll_period << endl;
+				//adm_dev->add_obj_polling(send,false);
+		}
+		catch (...)
+		{
 				//cout << "EventSubscriptionChangeCmd::execute(): failed to start polling !\n";
 				// nada
-			}
-			delete send;			
 		}
+			//delete send;			
+		//}
 	}
 
         CORBA::Any *ret = return_empty_any("EventSubscriptionChange");
