@@ -10,9 +10,15 @@
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.1 $
+// $Revision: 3.2 $
 //
 // $Log: seqvec.h,v $
+// Revision 3.2  2006/05/18 08:51:56  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
 // Revision 3.1  2005/01/13 08:30:00  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -149,9 +155,9 @@ namespace Tango
  */
 inline void operator<<(DevVarCharArray &lval,const vector<unsigned char> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -188,9 +194,9 @@ inline void operator<<(vector<unsigned char> &lval,const DevVarCharArray &rval)
  */
 inline void operator<<(DevVarShortArray &lval,const vector<short> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -226,9 +232,9 @@ inline void operator<<(vector<short> &lval,const DevVarShortArray &rval)
  */
 inline void operator<<(DevVarLongArray &lval,const vector<long> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -264,9 +270,9 @@ inline void operator<<(vector<long> &lval,const DevVarLongArray &rval)
  */
 inline void operator<<(DevVarFloatArray &lval,const vector<float> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -303,9 +309,9 @@ inline void operator<<(vector<float> &lval,const DevVarFloatArray &rval)
  */
 inline void operator<<(DevVarDoubleArray &lval,const vector<double> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -343,9 +349,9 @@ inline void operator<<(vector<double> &lval,const DevVarDoubleArray &rval)
  */
 inline void operator<<(DevVarBooleanArray &lval,const vector<bool> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -382,9 +388,9 @@ inline void operator<<(vector<bool> &lval,const DevVarBooleanArray &rval)
  */
 inline void operator<<(DevVarUShortArray &lval,const vector<unsigned short> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -422,9 +428,9 @@ inline void operator<<(vector<unsigned short> &lval,const DevVarUShortArray &rva
  
 inline void operator<<(DevVarULongArray &lval,const vector<unsigned long> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = rval[i];
 }
 
@@ -460,9 +466,9 @@ inline void operator<<(vector<unsigned long> &lval,const DevVarULongArray &rval)
  */
 inline void operator<<(DevVarStringArray &lval,const vector<string> &rval)
 {
-	long nb_elt = rval.size();
-	lval.length(nb_elt);
-	for (long i = 0;i < nb_elt;i++)
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
 		lval[i] = CORBA::string_dup(rval[i].c_str());
 }
 
