@@ -8,9 +8,30 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.14 $
+// $Revision: 3.17 $
 //
 // $Log: tango_const.h,v $
+// Revision 3.17  2006/05/23 12:02:06  jensmeyer
+// Added a reconnection delay of 1000ms not to send to many unnecessary
+// requestes to the database. Modified Connection::reconnect().
+//
+// Revision 3.16  2006/05/18 14:29:12  jensmeyer
+// Changed the handling of the event period for periodic and archiving events.
+// The event period is now stored as the configured value and no longer
+// multilplied by the factor DELTA_PERIODIC.
+// The calculation of the precision to fire periodic events has moved from
+// the attribbute to the push methods in the event supplier.
+//
+// For event periods < 5000 ms a precision of 2% was kept, but for
+// times < 5000 ms a fixed precision of 100 ms was added.
+// This avoids periodic events with a long event period to be fired to early.
+//
+// Revision 3.15  2006/05/18 08:51:56  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
 // Revision 3.14  2005/07/28 07:34:41  taurel
 // - Fix some incompatibility between 5.1 and 5.2
 //
@@ -207,7 +228,7 @@ namespace Tango
 // Some general interest define
 //
 
-#define		TgLibVers			"5.3.0"
+#define		TgLibVers			"5.5.0"
 
 #define		DevVersion			3
 #define		DefaultMaxSeq			20
@@ -247,14 +268,17 @@ namespace Tango
 #define		TG_IMP_MINOR_DEVFAILED		11
 #define		TG_IMP_MINOR_NON_DEVFAILED	12
 
+#define		TANGO_PY_MOD_NAME		"_PyTango.pyd"
+
 //
 // Event related define
 //
 
 #define 	EVENT_HEARTBEAT_PERIOD 		10
 #define 	EVENT_RESUBSCRIBE_PERIOD 	600
-#define		DEFAULT_EVENT_PERIOD		1000.0
-#define		DELTA_PERIODIC			0.98
+#define		DEFAULT_EVENT_PERIOD		1000
+#define		DELTA_PERIODIC				0.98  // Using a delta of 2% only for times < 5000 ms
+#define     DELTA_PERIODIC_LONG		100   // For times > 5000ms only keep a delta of 100ms
 #define		HEARTBEAT			"Event heartbeat"
 
 //
@@ -264,6 +288,12 @@ namespace Tango
 #define		CLNT_TIMEOUT_STR		"3000"
 #define		CLNT_TIMEOUT			3000
 
+//
+// Time to wait before trying to reconnect after
+// a connevtion failure
+//
+#define 		RECONNECTION_DELAY		1000   //ms. Only try to reconnect every second
+
 //
 // Max transfer size 8 MBytes (in byte). Needed by omniORB
 //
