@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: utils.cpp,v 3.13 2005/08/09 15:47:34 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: utils.cpp,v 3.19 2006/05/25 10:21:23 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -11,9 +11,30 @@ static const char *RcsId = "$Id: utils.cpp,v 3.13 2005/08/09 15:47:34 jensmeyer
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.13 $
+// $Revision: 3.19 $
 //
 // $Log: utils.cpp,v $
+// Revision 3.19  2006/05/25 10:21:23  taurel
+// - Create the per-thread data for main thread in server_init() method instead of server_run(). It was the reason of a seg fault when DS aborted in its startup sequence due to device init_device() method throwing exception
+//
+// Revision 3.18  2006/05/18 09:31:17  taurel
+// - Oups a bug in C++ server introduced by the port of Python DS to windows.Now, it  is fixed.
+//
+// Revision 3.17  2006/05/18 08:51:56  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
+// Revision 3.16  2006/02/20 08:16:25  taurel
+// - Add a new method in DeviceClass to delete a device at run-time
+//
+// Revision 3.15  2006/02/17 16:49:57  jensmeyer
+// Corrections when porting to VC7 under windows
+//
+// Revision 3.14  2006/01/20 08:38:39  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 3.13  2005/08/09 15:47:34  jensmeyer
 // Changed the clean-up mechanism when shutting down
 // the ORB. Destroy ORB when returning from run().
@@ -248,6 +269,8 @@ static const char *RcsId = "$Id: utils.cpp,v 3.13 2005/08/09 15:47:34 jensmeyer
 
 #include <omniORB4/omniInterceptors.h>
 
+omni_thread::key_t key_py_data;
+
 namespace Tango
 {
 
@@ -269,6 +292,7 @@ bool Util::_service = false;
 // in blackbox.cpp
 //
 
+
 omni_thread::key_t key;
 
 
@@ -446,8 +470,12 @@ void Util::effective_job(int argc,char *argv[])
 // and allocate a key for per thread specific storage
 //
 
-		omniORB::getInterceptors()->serverReceiveRequest.add(get_client_addr);
+		omni::omniInterceptors *intercep = omniORB::getInterceptors();
+		intercep->serverReceiveRequest.add(get_client_addr);
+		intercep->createThread.add(create_PyPerThData);
+		
 		key = omni_thread::allocate_key();
+		key_py_data = omni_thread::allocate_key();
 				
 //
 // Get some CORBA object references
@@ -1532,6 +1560,19 @@ void Util::server_already_running()
 
 void Util::server_init(bool with_window)
 {
+
+//
+// Even if we are not in a Python DS, we have to create the per-thread
+// PyData object. For Python DS, this is done in the Python_init method
+// defined in the binding
+//
+
+	omni_thread *th = omni_thread::self();
+	if (is_py_ds() == false)
+	{
+		th->set_value(key_py_data,new Tango::PyData());
+	}
+	
 #ifdef WIN32
 	if (with_window == true)
 	{
@@ -1547,17 +1588,17 @@ void Util::server_init(bool with_window)
 //
 
 
-#ifndef TANGO_HAS_LOG4TANGO
+	#ifndef TANGO_HAS_LOG4TANGO
     		cout.rdbuf(ds_window->get_output_buffer());
     		cout_tmp.rdbuf(ds_window->get_output_buffer());
-#endif 
+	#endif 
 	}
-#ifdef TANGO_HAS_LOG4TANGO //MODIF-NL
+	#ifdef TANGO_HAS_LOG4TANGO //MODIF-NL
   	else
 	{
     		ds_window = 0;
   	}
-#endif
+	#endif
 
 	if (_win == true)
 	{
@@ -1568,34 +1609,24 @@ void Util::server_init(bool with_window)
 	}
 	else
 	{
+#endif /* WIN 32 */
 	
 //
 // Initialise main class
 //
 
 		DServerClass::init();
-
+		
 //
 // Configure polling from the polling properties
 //
 		
 		polling_configure();
+		
+#ifdef WIN32
 	}
+#endif /* WIN32 */
 	
-#else
-
-//
-// Initialise main class
-//
-
-	DServerClass::init();
-	
-//
-// Configure polling from polling properties
-//
-
-	polling_configure();
-#endif
 }
 
 //+----------------------------------------------------------------------------
@@ -1608,7 +1639,10 @@ void Util::server_init(bool with_window)
 
 void Util::server_run()
 {
-	
+
+	omni_thread *th = omni_thread::self();
+	int th_id = th->id();
+		
 //
 // For Windows in a non-MSDOS window, start the ORB in its own thread. The main
 // thread is used for windows management.
@@ -1641,7 +1675,7 @@ void Util::server_run()
 					orb->run();
 					server_cleanup();
 				}
-				catch (CORBA::Exception &e)
+				catch (CORBA::Exception &)
 				{	
 					server_cleanup();
 					throw;
@@ -1650,14 +1684,29 @@ void Util::server_run()
 		}			
 		else
 		{
+			cerr << "Ready to accept request" << endl;
+
+			if (th_id == 0)
+			{
+				omni_thread::value_t *tmp_py_data = th->get_value(key_py_data);
+				PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+				lock_ptr->Release();
+			}
 
 			//JM : 9.8.2005 : destroy() should be called at the exit of run()!
 			try
 			{
 				orb->run();
 				server_cleanup();
+
+				if (th_id == 0)
+				{
+					omni_thread::value_t *tmp_py_data = th->get_value(key_py_data);
+					PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+					lock_ptr->Get();
+				}
 			}
-			catch (CORBA::Exception &e)
+			catch (CORBA::Exception &)
 			{	
 				server_cleanup();
 				throw;
@@ -1667,14 +1716,29 @@ void Util::server_run()
 #else
 	cout << "Ready to accept request" << endl;
 	
+	if (th_id == 0)
+	{
+		omni_thread::value_t *tmp_py_data = th->get_value(key_py_data);
+		PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+		lock_ptr->Release();
+	}
+							
 	//JM : 9.8.2005 : destroy() should be called at the exit of run()!
 	try
 	{
 		orb->run();
 		server_cleanup();
+
+		if (th_id == 0)
+		{
+			omni_thread::value_t *tmp_py_data = th->get_value(key_py_data);
+			PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+			lock_ptr->Get();
+		}
+	
 	}
 	catch (CORBA::Exception &e)
-	{	
+	{
 		server_cleanup();
 		throw;
 	}
@@ -1695,7 +1759,6 @@ void Util::server_cleanup()
 //
 // Destroy the ORB
 //
-
 	if (_constructed == true)
 	{
 		orb->destroy();
@@ -1859,7 +1922,7 @@ DeviceImpl *Util::get_device_by_name(const string &dev_name)
 //
 // Throw exception if the device is not found
 //
-	
+
 	if ((i == nb_class) && (j == nb_dev))
 	{
 		TangoSys_OMemStream o;
@@ -2362,10 +2425,10 @@ void PollConfigureThread::run(void *ptr)
 					if (throw_ex == true)
 					{
 						TangoSys_OMemStream o;
-						o << "System property polled_xxx for device " << v_poll_cmd[i]->svalue[0].in() << " has wrong syntax" << ends;
-						Except::throw_exception((const char *)"API_BadConfigurationProperty",
-				        				o.str(),
-				        				(const char *)"PollConfigureThread::run_undetached()");
+						o << "System property polled_xxx for device " << v_poll_cmd[i]->svalue[0].in() << " has wrong syntax or not supported attribute/command" << ends;
+						Except::re_throw_exception(e,(const char *)"API_BadConfigurationProperty",
+									o.str(),
+									(const char *)"PollConfigureThread::run_undetached");
 					}
 				}
 
@@ -2373,9 +2436,9 @@ void PollConfigureThread::run(void *ptr)
 				if (nb_cmd > 1)
 				{		
 #ifdef WIN32
-				Sleep(sleeping_time);
+					Sleep(sleeping_time);
 #else
-				nanosleep(&to_wait,&inter);
+					nanosleep(&to_wait,&inter);
 #endif
 				}
 			}
@@ -2430,8 +2493,15 @@ void PollConfigureThread::run(void *ptr)
 				tg->get_polling_thread_object()->join(0);
 				tg->clr_poll_th_ptr();
       	   			tg->unregister_server();
+
+				omni_thread::self()->set_value(key_py_data,new PyData());				
+				omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
+				PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+				lock_ptr->Get();
+					
 				tg->get_dserver_device()->delete_devices();
 
+				lock_ptr->Release();
 				if (tg->_FileDb == true)
 				{
 					Database *db_ptr = tg->get_database();
@@ -2457,7 +2527,7 @@ void PollConfigureThread::run(void *ptr)
 #endif
 			}
 		}
-	}				
+	}
 }
 
 //+----------------------------------------------------------------------------
@@ -2520,7 +2590,6 @@ void Util::trigger_attr_polling(Tango::DeviceImpl *dev,string &name)
 // still not executed
 //
 
-	int interupted;
 	TangoMonitor &mon = get_poll_monitor();
 	PollThCmd &shared_cmd = get_poll_shared_cmd();
 
@@ -2546,7 +2615,7 @@ void Util::trigger_attr_polling(Tango::DeviceImpl *dev,string &name)
 		TangoMonitor &dev_mon = dev->get_dev_monitor();
 		omni_thread *th = omni_thread::self();
 		
-		while (shared_cmd.trigger == true)
+/*		while (shared_cmd.trigger == true)
 		{
 
 //
@@ -2590,7 +2659,7 @@ void Util::trigger_attr_polling(Tango::DeviceImpl *dev,string &name)
 					        	(const char *)"Polling thread blocked !!!",
 					        	(const char *)"Util::trigger_attr_polling");
 			}
-		}
+		}*/
 	}
 	cout4 << "Thread cmd normally executed" << endl;
 }
@@ -2656,7 +2725,6 @@ void Util::trigger_cmd_polling(Tango::DeviceImpl *dev,string &name)
 // still not executed
 //
 
-	int interupted;
 	TangoMonitor &mon = get_poll_monitor();
 	PollThCmd &shared_cmd = get_poll_shared_cmd();
 
@@ -2682,7 +2750,7 @@ void Util::trigger_cmd_polling(Tango::DeviceImpl *dev,string &name)
 		TangoMonitor &dev_mon = dev->get_dev_monitor();
 		omni_thread *th = omni_thread::self();
 		
-		while (shared_cmd.trigger == true)
+/*		while (shared_cmd.trigger == true)
 		{
 		
 //
@@ -2726,7 +2794,7 @@ void Util::trigger_cmd_polling(Tango::DeviceImpl *dev,string &name)
 					        	(const char *)"Polling thread blocked !!!",
 					        	(const char *)"Util::trigger_cmd_polling");
 			}
-		}
+		}*/
 	}
 	cout4 << "Thread cmd normally executed" << endl;
 }
@@ -2841,6 +2909,16 @@ CoutBuf *Util::get_debug_object()
 
 void *Util::ORBWin32Loop::run_undetached(void *ptr)
 {
+//
+// Create the per thread data for the main thread
+//
+
+	omni_thread::self()->set_value(key_py_data,new Tango::PyData());
+	
+//
+// Create the DServer object
+//
+
 	try
 	{
 		DServerClass::init();
@@ -2914,4 +2992,36 @@ void clear_att_dim(Tango::AttributeValue_3 &att_val)
 	att_val.w_dim.dim_y = 0;
 }
 
+
+//
+// The function called by the interceptor on thread creation
+//
+
+void create_PyPerThData(omni::omniInterceptors::createThread_T::info_T &info)
+{
+	PyData *py_dat_ptr = new PyData();
+	omni_thread::self()->set_value(key_py_data,py_dat_ptr);
+	
+	info.run();
+	
+	omni_thread::self()->remove_value(key_py_data);
+	delete py_dat_ptr;
+	return;
+}
+
+AutoPyLock::AutoPyLock()
+{
+	omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
+	PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+	lock_ptr->Get();
+}
+	
+AutoPyLock::~AutoPyLock()
+{
+	omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
+	PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+	lock_ptr->Release();
+}
+	
+	
 } // End of Tango namespace
