@@ -8,9 +8,24 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.10 $
+// $Revision: 3.15 $
 //
 // $Log: utils.h,v $
+// Revision 3.15  2006/03/03 10:51:56  taurel
+// - Added a new python debugger flag
+//
+// Revision 3.14  2006/02/17 16:48:18  jensmeyer
+// Correction for porting to VC7 under windows.
+//
+// Revision 3.13  2006/01/20 08:41:32  taurel
+// - Only change in indentention...
+//
+// Revision 3.12  2006/01/20 08:33:28  taurel
+// - Added necessary changes to support Device server written in Python
+//
+// Revision 3.11  2005/12/16 17:41:12  jensmeyer
+// Added a check fro ATTR_WARNING quality.
+//
 // Revision 3.10  2005/06/29 08:31:19  taurel
 // - Last commit before release 5.2 ?
 //
@@ -234,6 +249,8 @@ class DServer;
 class AutoTangoMonitor;
 class Util;
 class EventSupplier;
+class PyLock;
+class CreatePyLock;
 
 #ifdef WIN32
 class CoutBuf;
@@ -241,6 +258,26 @@ class W32Win;
 #endif
 
 
+class PyLock
+{
+public:
+	PyLock() {}
+	virtual ~PyLock() {}
+	
+	virtual void Get() {}
+	virtual void Release() {}
+};
+
+class CreatePyLock
+{
+public:
+	CreatePyLock() {}
+	virtual ~CreatePyLock() {}
+	
+	virtual PyLock *create() {return new PyLock();}
+};
+
+
 //=============================================================================
 //
 //			The Util class
@@ -256,11 +293,14 @@ class W32Win;
 //
 //=============================================================================
 
+
 class UtilExt
 {
 public:
-	UtilExt():poll_mon("utils_poll"),ser_model(BY_DEVICE),only_one("process")
-	{shared_data.cmd_pending=false;shared_data.trigger=false;}
+	UtilExt():poll_mon("utils_poll"),ser_model(BY_DEVICE),only_one("process"),
+	 	  py_interp(NULL),py_ds(false),py_dbg(false)
+	{shared_data.cmd_pending=false;shared_data.trigger=false;
+	cr_py_lock = new CreatePyLock();}
 	
 	vector<string>		cmd_line_name_list;
 	
@@ -272,6 +312,11 @@ public:
 	SerialModel		ser_model;		// The serialization model
 	TangoMonitor		only_one;		// Serialization monitor
 	EventSupplier		*event_supplier;	// The event supplier object
+
+	void			*py_interp;		// The Python interpreter
+	bool			py_ds;			// The Python DS flag
+	CreatePyLock		*cr_py_lock;		// The python lock creator pointer
+	bool			py_dbg;			// Badly written Python dbg flag
 };
 
 /**
@@ -281,7 +326,7 @@ public:
  * one instance of this class and its constructor is not public.
  *
  * $Author: taurel $
- * $Revision: 3.10 $
+ * $Revision: 3.15 $
  */
  
 class Util
@@ -541,7 +586,19 @@ public:
 	int get_polling_thread_id() {return ext->poll_th_id;}
 	void create_event_supplier();
 	
+	
+	void *get_py_interp() {return ext->py_interp;}
+	void set_py_interp(void *ptr) {ext->py_interp = ptr;}
 
+	bool is_py_ds() {return ext->py_ds;}
+	void set_py_ds() {ext->py_ds=true;}
+	
+	bool is_py_dbg() {return ext->py_dbg;}
+	void set_py_dbg() {ext->py_dbg=true;}
+
+	void set_py_lock_creator(CreatePyLock *py) {ext->cr_py_lock = py;}
+	CreatePyLock *get_py_lock_creator() {return ext->cr_py_lock;}
+			
 /**@name Database related methods */
 //@{
 /**
@@ -1035,7 +1092,8 @@ public:
 				(*back)[0].r_dim.dim_y = 0;
 
 				if ((qu == Tango::ATTR_VALID) || 
-				    (qu == Tango::ATTR_ALARM) || 
+				    (qu == Tango::ATTR_ALARM) ||
+				    (qu == Tango::ATTR_WARNING) ||  
 				    (qu == Tango::ATTR_CHANGING))
 				{
 				
@@ -1384,6 +1442,68 @@ public:
 };
 
 
+
+
+
+//
+// For thread creation interceptor (Python device servers
+//
+
+void create_PyPerThData(omni::omniInterceptors::createThread_T::info_T &);
+
+
+class PyData: public omni_thread::value_t
+{
+public:
+	PyData():rec_state(false),rec_status(false)
+	{
+		try
+		{
+			Util *tg = Util::instance(false);
+			CreatePyLock *Creator = tg->get_py_lock_creator();
+			PerTh_py_lock = Creator->create();
+		}
+		catch(Tango::DevFailed &) {PerTh_py_lock=NULL;}
+	}
+	
+	~PyData()
+	{
+		if (PerTh_py_lock != NULL)
+			delete PerTh_py_lock;
+	}
+
+	DevVarCharArray			PerTh_dvca;
+	DevVarShortArray		PerTh_dvsha;
+	DevVarLongArray			PerTh_dvla;
+	DevVarFloatArray		PerTh_dvfa;
+	DevVarDoubleArray		PerTh_dvda;
+	DevVarUShortArray		PerTh_dvusa;
+	DevVarULongArray		PerTh_dvula;
+	DevVarStringArray 		PerTh_dvsa;
+	DevVarLongStringArray		PerTh_dvlsa;
+	DevVarDoubleStringArray		PerTh_dvdsa;
+	
+	string				PerTh_string;
+	DevFailed			PerTh_df;
+	vector<string>			PerTh_vec_str;
+	vector<double>			PerTh_vec_db;
+	DevErrorList			PerTh_del;
+	
+	bool				rec_state;
+	bool				rec_status;
+	
+	PyLock				*PerTh_py_lock;
+};
+
+
+class AutoPyLock
+{
+public:
+	AutoPyLock();
+	~AutoPyLock();
+};
+
+
 } // End of Tango namespace
 		     
 #endif /* UTILS */
